# EULA_check_agree.py


import bpy  
import sys
import os
import webbrowser
import shutil

'''
This script may look *unbelievably complex* to just launch a popup -- if you find a simpler way, please let cnm know!
Problem with usual ways of doing this is that when Blender is starting up and when it is doing add-on installs
it goes into a restricted context mode. The normal ways of doing popups then fail -- with restriction/context/view_layer
errors. The script below manages to set up a valid context regardless, and it works.
'''




one_shot_flag   = False                  


                                            # get visibility for the package and modules --- append sys path
plugin_dir      = os.path.dirname(__file__)
if plugin_dir not in sys.path:  sys.path.append( plugin_dir)


from logger_module    import   Logger       # setup logger
logger          = Logger.get_logger()       
   


    
    
    

def do_EULA_popup():
    logger.info(f'***** popup flag={one_shot_flag} *****\n\n')
    
    if one_shot_flag:
        '''
        if "EULAAgreementOperator" not in bpy.types.__dict__:
            try:
                bpy.utils.register_class(EULAAgreementOperator)
            except Exception as ex:
                logger.warning(f'could not register EULAAgreementOperator') 
        '''
        
        bpy.app.timers.register( trigger_EULA_dialog, first_interval=0.1)





class EULAAgreementOperator( bpy.types.Operator):           # get user agreement to EULA, else shut down Blender
    
    bl_idname = "wm.eula_agreement_dialog"
    bl_label  = "Pristina (TM)-- End User License Agreement (EULA)"
    
    def execute(self, context):
        
        logger.info( f"\n\n\t***** User has agreed EULA *****\n\n") 

        launch_main_app() 

        logger.info( f"\n\n\t***** Returned from launch_main_app() *****\n\n") 


        import install_completed____popup_dialog  as done
        
        try:
            bpy.utils.register_class( done.ConfirmShutdownOperator2)
            logger.info( f'registered ConfirmShutdownOperator2')
        except Exception as ex:
            logger.error( f'could not register ConfirmShutdownOperator2  {ex}')
        done.launch_popup = True
        
        logger.info(f'Launching the completed popup')
        done.launch_install_completed_popup()        
        
        return {'FINISHED'}
        

    def cancel(self, context):    
        logger.info( f"\n\n\t***** User has CANCELLED, cleaning up, unregistering and quiting Blender *****\n\n")    
        cleanup("Pristina_plugin")
        unregister()
        logger.info(f'All done')
        bpy.ops.wm.quit_blender()         
            
    
    def draw(self, context):
        layout = self.layout
        layout.label(text = " ")
        layout.label(text = "Please read the End User License Agreement")
        layout.operator(    "wm.open_eula", text="Read EULA")
        layout.label(text = " ")
        layout.label(text = "and click 'OK' to agree to it.")
    
    
    def invoke(self, context, event):
        logger.info("EULAAgreementOperator.invoke() called, showing confirmation dialog...")      
        
                                                            
        return context.window_manager.invoke_props_dialog(self)         # Open a modal dialog  





class OpenEULAOperator(bpy.types.Operator):                 # Operator to open the EULA link in the default web browser
    bl_idname = "wm.open_eula"
    bl_label = "Open EULA in Browser"
    bl_description = "Opens EULA in your browser"

    def execute(self, context):
        eula_url = "https://www.pristina.ai/eula/"       # todo Replace with link to EULA eg: "https://www.pristina.ai/eula"
        webbrowser.open(eula_url)
        return {'FINISHED'}




def trigger_EULA_dialog():                                  # trigger the modal dialog safely in the Preferences window context
    global one_shot_flag
    
    if one_shot_flag:
        one_shot_flag = False                               # *** Reset the flag immediately to avoid repeated triggering  ***
        logger.info("Invoking EULA dialog in Preferences context...")        
        preferences_area = None                             # Make sure the dialog is associated with the Preferences window context

        for window in bpy.context.window_manager.windows:   # Find the Preferences window context
            for area in window.screen.areas:
                if area.type == 'PREFERENCES':
                    preferences_area = area
                    break

        if preferences_area:
            with bpy.context.temp_override(window=window, area=preferences_area):
                bpy.ops.wm.eula_agreement_dialog('INVOKE_DEFAULT')
        else:
            logger.info("Preferences area not found, cannot trigger EULA dialog.")
    
    return None                                             # Stops the timer from repeating




    
# --------------------------------------------------------------  WHEN WE GET EULA 'OK'
    
    
def get_exr_source_and_dest():
    exr_file    = "check_normal_pristina.exr"
    source      = os.path.join( plugin_dir, exr_file)
     
    if not os.path.exists( source):        # .exr *must* be included in plugin zip before distribution to users
        
        raise RuntimeError( f"Could not find .exr {source}")    # dev only, it should never happen to user

    install_dir = bpy.utils.resource_path('LOCAL')
    matcap_dir  = os.path.join( install_dir, "datafiles", "studiolights", "matcap")

    if not os.path.exists( matcap_dir):  os.makedirs( matcap_dir)

    dest = os.path.join( matcap_dir, exr_file)
    
    return source, dest
    
    
            
def  launch_main_app():     # This launches the install proper, with .exr copy done first
    
    source, dest  = get_exr_source_and_dest()
    
    try:
        # DURING INSTALL PROCESS (WITH WRITE PERMISSION ALREADY CHECKED), THIS WILL SUCCEED:
        shutil.copy2(source, dest)      # shutil.copy2 preserves file metadata  -- do for real this time   (not as a test)
        logger.info(f"\n\nCopied \n{source}\n>>> to >>>\n{dest}\n\n")   # operations were successful
    
    except Exception as ex:
        # NORMAL USAGE (NOT INSTALLATION) ABOVE FAILS -- AS WE DO NOT HAVE ELEVATED PERMISSIONS
        # TO COPY .EXR FILE INTO MATCAP FOLDER (PROTECTED AREA)
        # THIS .EXR FILE IS ALREADY IN PLACE, SO WE DON'T CARE IF THIS COPY FAILS   (EAFP APPROACH)
        pass                                                  
    
    logger.info( f'importing main')
    import main    
    logger.info( f'\n\n\t***** success in importing main (next call main.register()) *****\n\n')
    main.register()
    
    logger.info( f'check exr in place  {os.path.exists(dest)}  size={os.path.getsize( dest)}')

    logger.info( f'\n\n\t***** success in registering main *****\n\n')

        
    

#---------------------------------------------------------------------------------------------------------------------------          
# example of plugin directory=   C:\Users\cex\AppData\Roaming\Blender Foundation\Blender\4.1\scripts\addons\Pristina_plugin



def delete_addon_files( addon_name, addon_path):
    if os.path.exists( addon_path):
        try:
            shutil.rmtree( addon_path)
            logger.info( f"Deleted addon folder: {addon_path}")
        except Exception as e:
            logger.info( f"Failed to delete addon folder: {e}")
    else:
        logger.info( f"Addon folder {addon_path} does not exist.")




def remove_cached_modules( addon_name):
    for module_name in list( sys.modules.keys()):
        if addon_name in module_name:
            del sys.modules[ module_name]
            logger.info( f"Removed cached module: {module_name}")




def remove_addon_from_preferences( addon_name):
    prefs = bpy.context.preferences
    
    for addon in prefs.addons:                  # Check if the addon is installed and enabled
        # logger.critical( f'addon={addon}')    # checking what this list of structures looks like
        if addon.module == addon_name:
            prefs.addons.remove( addon)  # Remove the addon from preferences
            logger.info(f"Removed addon reference from preferences: {addon_name}")
            break    # if break is never reached, then the else: block becomes active
    else:
        logger.info(f"No references to {addon_name} found in preferences.")


                
# ------------------------------------------
             
    
def  cleanup( addon_name):                              # individual tries, so one fail does not stop others
    
    try:            
        delete_addon_files( addon_name, plugin_dir)     # delete Blender's copy of the plugin modules   
    except Exception as ex:
        logger.exception( f'{ex}')
        
        
    try:    
        remove_cached_modules( addon_name)              # remove any modules cached by Blender  
    except Exception as ex:
        logger.exception( f'{ex}')
        
        
    try:    
        remove_addon_from_preferences( addon_name)      # find any Pristina mentions in prefs, and remove                                              
    except Exception as ex:
        logger.exception( f'{ex}')        
            
        
        

                    
    
    
    
 
# --------------------------------------------------------------  REGISTRATION
 
 
registration_classes = [
    OpenEULAOperator,
    EULAAgreementOperator, 
]


def register():
    global one_shot_flag  

    logger.info('EULA_check_agree  register() called')    

    for cls in registration_classes:
        
        try:
            bpy.utils.unregister_class(cls)
            logger.info(f'Unregistered {cls}')
        except Exception as ex:       
            pass     
                
        try:    
            bpy.utils.register_class( cls)                                    
            logger.info( f'registered  {cls}')
        except Exception as ex:
            logger.info(f'---------------Registration for {cls} failed: {ex}')
            pass      # in case comment out/remove logger
            
    logger.info( f'launching eula popup')    
    one_shot_flag     = True                                # set True to fire once, first firing clears (sets 
                                                            # to False) to prevent multiple launches
    do_EULA_popup()    

                
        

def unregister():
    logger.info('EULA_check_agree   unregister() called')  
    for cls in reversed( registration_classes):
        try:
            bpy.utils.unregister_class( cls)
            logger.info(f'UNregistered  {cls}')
        except Exception as ex:
            pass     
                
    
     
   