# compositor_setup.py

# from compositor_setup import  CompositorSetup

import bpy



class CompositorSetup():
    def __init__(self):
        # Ensure we're using nodes in the compositor
        bpy.context.scene.use_nodes = True
        self.tree                   = bpy.context.scene.node_tree
        # Initialize an empty list to keep track of created nodes specifically by this instance
        self.created_nodes          = []

    def setup_nodes(self, fileoutput):
        # Create Render Layers node
        render_layers_node          = self.tree.nodes.new(type='CompositorNodeRLayers')
        render_layers_node.location = (-300, 0)
        self.created_nodes.append(render_layers_node)
        
        # Create Invert node
        invert_node                 = self.tree.nodes.new(type='CompositorNodeInvert')
        invert_node.location        = (0, 0)
        self.created_nodes.append(invert_node)
        
        # Create Normalize node
        normalize_node              = self.tree.nodes.new(type='CompositorNodeNormalize')
        normalize_node.location     = (300, 0)
        self.created_nodes.append(normalize_node)
        
        # Create File Output node and set the output filename
        file_output_node            = self.tree.nodes.new(type='CompositorNodeOutputFile')
        file_output_node.location   = (600, 0)
        file_output_node.base_path  =  fileoutput#"//output"  # Specify the output directory relative to the .blend file
        # Here we set the output filename to 'pristina_depth'
        file_output_node.file_slots[0].path = "pristina_depth"
        self.created_nodes.append(file_output_node)
        
        # Link nodes together
        links = self.tree.links
        links.new(render_layers_node.outputs['Mist'], invert_node.inputs[1])  # Adjust the output if using a different pass
        links.new(invert_node.outputs[0], normalize_node.inputs[0])
        links.new(normalize_node.outputs[0], file_output_node.inputs[0])

    def cleanup_nodes(self):
        # Iterate over a copy of the list as we're modifying the original list inside the loop
        for node in self.created_nodes[:]:
            if node.name in self.tree.nodes:  # Check for the node's existence by its name
                self.tree.nodes.remove(self.tree.nodes[node.name])  # Remove node from the node tree
            self.created_nodes.remove(node)  # Remove the node from the tracking list


#        # Set the output filename and connect nodes
#        filename = object_name + "_mask"
#        file_output_node.base_path = self.output_folder  # Set your desired output folder
#        file_output_node.file_slots[0].path = filename
############################ End DEPTH setup ####################################
