# ensure_exr.py

# cd "C:\Users\cex\deep\FINZ\ipynb\Project_Pristina\Distro_Blender\Pristina_plugin"
# python ensure_exr.py



import sys
import os
import shutil

EXR_OK_GO_AHEAD  = 0

EXR_PERMISSION_ERROR = 1

EXR_WAITING_FOR_USER_TO_CONFIRM_QUIT = 2


try:
    import bpy
    install_dir = bpy.utils.resource_path('LOCAL')
    plugin_dir  = os.path.dirname(__file__)
    IN_BLENDER  = True
    
except:     # test / dev mode
    install_dir = r"C:\Program Files\Blender Foundation\Blender 4.1\4.1"
    plugin_dir  = r"C:\Users\cex\AppData\Roaming\Blender Foundation\Blender\4.1\scripts\addons\Pristina_plugin"
    IN_BLENDER  = False
    
    


# get visibility for the package and modules --- append sys path
if plugin_dir not in sys.path:
    sys.path.append( plugin_dir)

parent_dir      = os.path.dirname( plugin_dir)
if parent_dir not in sys.path:
    sys.path.append( parent_dir)


from logger_module    import   Logger

logger       = Logger.get_logger()      

FLAG_PERMISSION_ERROR_DO_POPUP     = False  # used by nested_ui_panels.py to make popup, 
                                            # notify user of need for elevated privileges
                                            # and ask user to restart as admin or root user
                                            # one time use -- when True it is acted on once, 
                                            # and immediately set back False to stop repeated actions
                                            
FLAG_PERMISSION_ERROR_STOP_INSTALL = False  # also set true if permission error, used 
                                            # in __init__.py by register() to stop install process 
                                            # which will be happening independently from the (async) popup
                                            
FLAG__GOT_EXR__OK_TO_PROCEED       = False  # initially false
                                            
#def get_go_ahead_flag():
 #   return FLAG__GOT_EXR__OK_TO_PROCEED
                                            
                                            
            
def ensure_exr_file():                      # this is called in __init__.py   in register()
    global FLAG_PERMISSION_ERROR_DO_POPUP
    global FLAG_PERMISSION_ERROR_STOP_INSTALL
    try:
        exr_file    = "check_normal_pristina.exr"
        source      = os.path.join( plugin_dir, exr_file)
         
        if not os.path.exists( source):
            raise RuntimeError( f"Could not find .exr {source}")    # .exr must be included in plugin zip before user uses app

        
        matcap_dir  = os.path.join( install_dir, "datafiles", "studiolights", "matcap")

        if not os.path.exists( matcap_dir):  os.makedirs( matcap_dir)

        dest = os.path.join( matcap_dir, exr_file)
        

        # ----------------------------------------------------
        # ***  Check if destination file exists already, and has the same size
        # ***  (if so, job already done) --  additional datetime check was found not to be reliable   
        # todo -- when making a new .exr file, just ensure its size is new, unique, so this method can tell it is different
        
        if os.path.exists( dest):
            source_stat = os.stat( source)
            dest_stat   = os.stat( dest)
            
            if (source_stat.st_size  == dest_stat.st_size ):       
                               
                logger.info(f"   .exr file is already in place and is up-to-date: \n{dest}")
                return  EXR_OK_GO_AHEAD     # meaning no permission error
        # ----------------------------------------------------
        

        # else, need to do the copy-over

        shutil.copy2(source, dest)      # shutil.copy2 preserves file metadata
                      
        logger.info(f"\n\nCopied \n{source}\n>>> to >>>\n{dest}\n\n")
        return     EXR_OK_GO_AHEAD          # operations were successful  permission error flag is still set False
        
       
       
    except PermissionError as ex:
        # pristina needs to copy new version of file into Blender, and needs elevated privileges
        logger.warning(f'\n\n\t*** Pristina install requires elevated privileges (admin or root) to avoid permission errors ***\n\n')
        logger.info(f'\n\n\t***** Now setting flag to do popup for user, then quit Blender *****\n\n')
        FLAG_PERMISSION_ERROR_DO_POPUP     = True
        FLAG_PERMISSION_ERROR_STOP_INSTALL = True
        # show_delayed_popup( "Pristina install requires elevated privileges to avoid permission errors\nPlease restart Blender as admin or root -- MUST NOW QUIT.")
        check_permission()    # start the process for notfifying user of requirement and seek confirmation for shutdown
        return  EXR_PERMISSION_ERROR    # tell caller -- init, register() -- to stop install immediately

    
    except Exception as ex:
        msg = f'copy exr unexpected exception: {ex}'
        logger.exception( msg)     
        raise RuntimeError( msg)  #  this is then a dev error, not for the user
            
            
            
            





# ------------------------------------------------------------------------------------------ code to deal with user trying to install without having elevated permissions

# May look as if flag is not needed and can just call the popup, however, it is set false as soon as used, 
# to avoid repeated triggering  
         
def check_permission():                                        
    logger.info(f'\n\n\t*****permission error flag={FLAG_PERMISSION_ERROR_DO_POPUP} *****\n\n')
    if FLAG_PERMISSION_ERROR_DO_POPUP:
        logger.info("Diagnostic: Permission error detected, flag set to True.")
        bpy.app.timers.register(trigger_confirmation_dialog, first_interval=0.1)    # Register handler to defer the modal operator until the UI is ready




class ConfirmShutdownOperator(bpy.types.Operator):
    """Warn the user to restart Blender with elevated privileges."""
    # Modal operator to seek user confirmation, then shut down Blender

    bl_idname = "wm.confirm_shutdown_dialog"
    bl_label  = "To install restart Blender as Admin or Root"
    
    def execute(self, context):
        logger.info("Diagnostic: ConfirmShutdownOperator.execute() called, shutting down Blender...")
        attempt_clean_uninstall()
        bpy.ops.wm.quit_blender()
        return {'FINISHED'}
    
    def invoke(self, context, event):
        logger.info("Diagnostic: ConfirmShutdownOperator.invoke() called, showing confirmation dialog...")
        return context.window_manager.invoke_confirm(self, event)






# Function to trigger the modal dialog safely in the Preferences window context
def trigger_confirmation_dialog():
    
    global FLAG_PERMISSION_ERROR_DO_POPUP
    
    if FLAG_PERMISSION_ERROR_DO_POPUP:
        # Reset the flag to avoid repeated triggers
        FLAG_PERMISSION_ERROR_DO_POPUP = False
        
        logger.info("Diagnostic: Invoking custom confirmation dialog in Preferences context...")

        # Make sure the dialog is associated with the Blender Preferences window
        preferences_area = None

        # Find the Preferences window context
        for window in bpy.context.window_manager.windows:
            for area in window.screen.areas:
                if area.type == 'PREFERENCES':
                    preferences_area = area
                    break

        if preferences_area:
            with bpy.context.temp_override(window=window, area=preferences_area):
                bpy.ops.wm.confirm_shutdown_dialog('INVOKE_DEFAULT')
        else:
            logger.info("Diagnostic: Preferences area not found, cannot trigger confirmation dialog.")
    
    return None  # Stops the timer from repeating



def attempt_clean_uninstall():
    """Attempt to uninstall the plugin and remove any installed files."""
    try:
        # Unregister the plugin to remove Blender components
        Pristina_plugin.unregister()       # method is in __init__.py so call with package name

        # Attempt to remove any installed files
        plugin_dir = os.path.dirname( __file__)  # Plugin's directory

        # Assuming the plugin installs files in a subdirectory 'installed_files'
        installed_dir = os.path.join( plugin_dir, "installed_files")

        if os.path.exists( installed_dir):
            shutil.rmtree( installed_dir)
            logger.info(f"Diagnostic: Successfully removed installed files in {installed_dir}.")
        else:
            logger.info(f"Diagnostic: No installed files found in {installed_dir}.")
        
        logger.info("Diagnostic: Plugin successfully unregistered and uninstalled.")

    except Exception as e:
        logger.info(f"Error: Failed to clean uninstall the plugin due to: {e}")



            


if __name__ == "__main__":
    ensure_exr_file()
    
    
'''
while True:
    if check() == 'perm_error':
        bpy.ops.wm.quit_blender()
    if check() == 'ok_go_ahead':
        break
    if check() == 'waiting_user_confirmation':
        continue


'''    