# fifo_queue_module.py

# import fifo_queue_module 

import bpy
import time
import queue
import threading
from   sessions                       import   SessionManager
from   upload_task_wait_complete      import   upload_task
from   logger_module                  import   Logger
import popup_for_job_fail 
import shutil
import os
from log_uploader_REUSED              import   LogUploader



logger      = Logger.get_logger()      ####################  global, is set up on load 



fifo_queue  = queue.Queue()                 # Define a thread-safe FIFO queue

stop_event  = threading.Event()             # Event to signal stopping of all threads (executor threads and print thread)
                                            # NOT CURRENTLY IMPLEMENTED    TODO DECIDE IF NEEDED or 'yagni'
                                            # status is checked in wait_for_complete()


# called from ot_render_pristina module

def new_job( pristinaFilePath, userEmail, userPassword, context,  timeout_seconds):    # Function to handle new jobs without blocking

    logger.debug(f'\n\n\t***** starting new thread for pfpath={pristinaFilePath} *****\n\n')
        
    threading.Thread(   target = upload_task,           # in   upload_task_wait_complete.py
    
                        args   = (  pristinaFilePath, 
                                    userEmail,  
                                    userPassword,  
                                    context,  
                                    fifo_queue,
                                    timeout_seconds ), 
                                    
                        daemon = True ).start()         # daemon=True should stop any active threads when Blender exits
                        
                        
    
def queue_size():
    return fifo_queue.qsize()    


def list_queue():
    with fifo_queue.mutex:
        return list( fifo_queue.queue)


def pop_queue():
    if not fifo_queue.empty():
        return fifo_queue.get()
    return None

'''
def print_queue_contents():
    previous_contents = None
    while not stop_event.is_set():                  # used during shutdown to allow thread to complete
        current_contents = list_queue()
        if current_contents != previous_contents:
            formatted = ', '.join( f"({id}, '{status}')" for id, status in current_contents )
            logger.info(  f"FIFO Queue contents (not the same as jobs in progress)\n\t[{formatted}]" )
            previous_contents = current_contents
            if stop_event.is_set():                 # don't wait for sleep before returning on stop
                break
        time.sleep( 0.3 )
         
    logger.info(f'thread stopped')

# Start the thread to display queue contents
#printer_thread = threading.Thread( target=print_queue_contents)
#printer_thread.daemon = True                        # Allow the thread to exit when the main program exits
#printer_thread.start()

'''         



def check_fifo_queue( pristina_properties):
    from task_manager       import  TaskManager
    from nested_ui_panels   import  PRISTINA_PT_MainPanel
    
    logger.debug( f'...... fifo check: timer-heartbeat\n\n\t ******* now there are {count_pristina_threads()} pristina threads running ******\n\n' )        
    
    if not fifo_queue.empty():                                          # if there are jobs in queue, then process them 
                                                                        # heartbeat timer regularly calls this checker
    
        try:            
            while not fifo_queue.empty():                               # process every item currently available
            
                task_id, status = fifo_queue.get_nowait()               # unpack [task id, status] from queue 
                logger.info( f'got fifo message:  {task_id}  {status}')
                
                if status == 'SYNC_DONE':
                    PRISTINA_OT_SyncResults.revert_to_idle()            # change button text from 'syncing' back to 'sync'
                    continue
                    
                                
                if status in ['COMPLETE', 'FAIL']:                      # *** IMMEDIATELY END THREADS NOT IN USE ***
                    if status == 'FAIL':                                # to test popup set as "if True: #status == 'FAIL':  "
                        logger.info( f'\n\n\t***** Notifying user that task {task_id} has failed *****\n\n')
                        popup_for_job_fail.notify( task_id)
                        
                        result = LogUploader.on_job_fail(task_id)       # currently ignore result, 
                                                                        # not critical (mainly used for testing)
                    
                    end_thread_for_task( task_id)
                
                result  = TaskManager.update_task_status( task_id, status)
                logger.debug(f'update task status result={result}')
                
                
        except queue.Empty:
            pass                                                        # If the queue is empty, do nothing, 
                                                                        # just carry on to code below
        except Exception as ex:
            logger.exception(f'\n\n\t***** unexpected exception {ex} *****\n\n')
            # todo decide what to do here --- is this survivable -- if unexpected then probably not
            # --- make overview chart of stages of usage and recoverable error stages
            
            
        # now dealt with one or more jobs, need to update the UI
        # 'jobs_in_progress' custom property and trigger refresh of UI panel
        
        jip                                    = TaskManager.count_all_pending_and_actioned()        
        pristina_properties.jobs_in_progress   = jip
        logger.info( f'\n\n\t***** number of jobs in progress now ={jip} *****\n\n')
        
        if jip==0:
            # now there are *no jobs in progress*                        
            output_dir     = bpy.path.abspath( pristina_properties.file_output)  
            tmp_dir        = os.path.join( output_dir, "Pristina_tmp")
            
            remove_directory_tree( tmp_dir)                 # delete the 'Pristina_tmp' folder + all contents
                                        
            PRISTINA_PT_MainPanel.redraw_panel()
            Timer_for_FIFO_checking.stop()                  # stop the checking timer while there are no jobs in progress
            logger.debug( f'\n\n\t ******* now there are {count_pristina_threads()} pristina threads running ******\n\n' ) 
            
            try:
                logger.debug(f'try and sign out of Supabase (until next job live)')
                this_thread_id  = SessionManager.get_thread_id_with_name( 'Main' )                                 
                SessionManager.logout( this_thread_id)      # shut the Main session down while no jobs in progress
                
                
                
            except Exception as ex:
                logger.exception( f'Exception while signing out of Supabase: {ex}')
                pass   # todo decide what to do next if this occurs 
                       # ?? warn user to right click taskbar and close-all, and or check windows task mgr
                       # and end blender background process if still running (very rare now, but cannot guarantee)
                       # or put in faq?
                                                        
            
              
    

def remove_directory_tree( directory_path):    
                       # !!! immediate removal causes problems because of things 
                       # still going on in the background, therefore:    
    time.sleep(1)      # Delay for 1 second to ensure background file processes haved completed
    try:
        if os.path.exists(directory_path):
            shutil.rmtree(directory_path)
            logger.info(f"Successfully removed temp dir: {directory_path}")
        else:
            logger.error(f"Temp dir does not exist: {directory_path}")
            
    except Exception as ex:
        logger.error(f"Error trying to remove dir: {ex}")   # log only, otherwise ignore, not mission critical
        


def count_pristina_threads():                 
    count = 0
    for thread in threading.enumerate():    
        if thread.name.startswith( 'pristina_thread_id_'): 
            count += 1
    return count
        



def end_thread_for_task( task_id):
    # get of list of threads currently running, find pristina ones, 
    # find one with matching task_id and shut it down

    for thread in threading.enumerate():
    
        if not thread.name.startswith('pristina_thread_id_'):   # skip non-pristina threads
            continue        
    
        thread_id = int( thread.name.split('_')[-1])            # NB name is str, so must convert to int for equality compare
        if not thread_id == task_id:                            # skip till get match
            continue
                
        logger.debug(f'\n\n\t***** task id={task_id} waiting for thread {thread.name} to join back  *****\n\n')
        if thread.is_alive():
            logger.debug(f'\n\n\t***** but task id={task_id} thread {thread.name} is still alive  *****\n\n')
            n = 3  # seconds
            time.sleep(n)
            if thread.is_alive():
                logger.debug(f'\n\n\t***** but task id={task_id} thread {thread.name} is still alive after (n) secs  *****\n\n')
                        
        thread.join()                                           # block till joined
        logger.info(  f'\n\n\t *****************************************************************************\n' +
                        f'\n\t ***** Thread for task {task_id} joined back OK  --- this is a very good thing\n' +
                        f'\n\n\t ******* now there are {count_pristina_threads()} pristina job threads running ******\n\n' )        
        return
                
    logger.debug(f'\n\n\t***** thread for id={task_id} not found in current threads (ok -- already ended itself) *****\n\n')
    




class Timer_for_FIFO_checking:  # uses class methods, singleton like, no need to instantiate object to call methods

    interval            = 3.0
    timer_running       = False
    pristina_properties = None


    @classmethod
    def start( cls, pristina_properties):
        if cls.timer_running:  # may well be running already, don't try and start again
            logger.debug("Timer already running, not starting again")
            return       
        cls.pristina_properties = pristina_properties
        cls.timer_running       = True
        bpy.app.timers.register( cls.check_fifo_now)
        logger.info( f'\n\n\t*****  11111111111111  FIFO check timer started *****\n\n')



    @classmethod
    def check_fifo_now( cls):
        #logger.debug("Entering check_fifo_now")
        if not cls.timer_running:
            #logger.debug(  "Timer not running, exiting check_fifo_now")
            logger.info( f'\n\n\t***** 00000000000000  FIFO check timer stopped *****\n\n\n\n')
            
            return None             # **This is all that is needed** to completely unregister the timer 
                                    # ie returning 'None' as time interval to next run
                                    # https://docs.blender.org/api/current/bpy.app.timers.html
                                    # any further attempt to unregister this timer will raise an exception
        try:
            # logger.debug("Timer is running, checking FIFO queue")

            # ''''''''''''''''''''''''''''''''''''''''''''''''
            check_fifo_queue( cls.pristina_properties)          # Payload, whole purpose of this timer 

            # ''''''''''''''''''''''''''''''''''''''''''''''''
            
        except Exception as e:
            logger.exception(f"\n\n\t***** Exception in check_fifo_queue: {e} *****\n\n")
        
        #logger.debug("Exiting check_fifo_now")
        return cls.interval         # Return interval (in secs) before being called again



    @classmethod
    def stop( cls):
        logger.debug(f'\n\n==================== timer stop called when timer_running={cls.timer_running}\n\n')
        if cls.timer_running:
            #logger.debug(f'\n\n==================== unregistering blender timer for check_fifo\n\n')
            cls.timer_running = False
            #try:
             #   bpy.app.timers.unregister( cls.check_fifo_now)
              #  logger.info( f'\n\n\t***** 00000000000000  FIFO check timer stopped and unregistered *****\n\n')
            #except Exception as e:
             #   logger.error(f"Exception while unregistering timer: {e}")

        # Clear properties even if the timer was not running
        #cls.pristina_properties = None
        #logger.info("Timer properties cleared")

