# file_utils.py


# from file_utils   import  append_timestamp_to_filename
# from file_utils   import  download_and_unzip
# from file_utils   import  PristinaZipper
 
 
IN_BLENDER = True    # switch --- False for 'out of Blender' testing of sync.py module 
try:
    import bpy
except ImportError:
    IN_BLENDER = False
    
    
    
import os
import re
import shutil
import zipfile
from   datetime                                          import datetime
import time
from   sessions                                          import SessionManager
from   retrex_decorator___retry_with_exception_catching  import retrex

from log_uploader_REUSED                                 import LogUploader

from logger_module                                       import Logger
logger = Logger.get_logger()      ####################  global, is set up on load 




def append_timestamp_to_filename(original_filepath):
    # Split the original filepath into directory, filename without extension, and extension
    dirname, filename = os.path.split(original_filepath)
    filename_without_ext, file_ext = os.path.splitext(filename)
    
    # Generate a timestamp string
    timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
    
    # Concatenate the original filename with the timestamp and extension
    new_filename = f"{filename_without_ext}_{timestamp}{file_ext}"
    
    # Reconstruct the full filepath
    new_filepath = os.path.join(dirname, new_filename)
    
    return new_filepath
    
    
    
    

class PristinaZipper:
    def __init__(self, folder_path, output_folder_path, zip_file_name, pristina_file_name):
        self.folder_path        = folder_path
        self.output_folder_path = output_folder_path
        self.zip_file_name      = zip_file_name
        self.pristina_file_name = pristina_file_name

    def zip_folder(self):
        # Create the output folder if it does not exist
        os.makedirs(self.output_folder_path, exist_ok=True)

        # Path for the zip file
        zip_file_path           = os.path.join(self.output_folder_path, self.zip_file_name)
        pristina_file_path      = os.path.join(self.output_folder_path, self.pristina_file_name)

        # Creating the zip file
        with zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
            for root, dirs, files in os.walk(self.folder_path):
                for file in files:
                    file_path   = os.path.join(root, file)
                    arcname     = os.path.relpath(file_path, self.folder_path)
                    zipf.write(file_path, arcname)

        # Rename the .zip file to .pristina
        os.rename(zip_file_path, pristina_file_path)
    


def get_task_id( zip_path):  # eg. ['1485_results/original.png', '1485_results/Pristina-Result.png'] ==> task_id is: 1485
    task_id = 0
    with zipfile.ZipFile(zip_path, 'r') as zip_ref:        
        all_items = zip_ref.namelist()
        # print( all_items)
        first     = all_items[0]
        task_id   = first.split('_')[0]        
    return task_id




def get_jobname_and_dir( results_folder, taskId ):
    
    pattern = re.compile(r'^(.*)___ref_(\d+)$')      # match dirs in form *___ref_{int}
    
    # List all items in the local folder and filter for directories
    for directory_name in os.listdir( results_folder ):
        directory_path = os.path.join( results_folder, directory_name )
        
        # Ensure it's a directory and matches the pattern
        if os.path.isdir( directory_path ):
            match = pattern.match( directory_name )
            if match:
                jobname, found_taskId = match.groups()
                
                # Check if the found task ID matches the input task ID
                if found_taskId == str( taskId ):
                    job_folder = os.path.join( results_folder, directory_name)
                    return jobname, job_folder
    
    # If no match was found, return an error message
    raise ValueError( f"Task ID {taskId} not found in any directory under '{results_folder}'" )
    
  
  
  
def do_renames(  results_folder, task_id ):
    
    # from task_id, get the user_jobname that got saved in the job_folder name at start of job
    # this approach means the user_jobname is persisted even when the plugin may have been shut down before
    # it could previously receive results -- this way solves that problem
    
    
    user_jobname = 'Recovered'                                      # set defaults in case expected ones are not found for any reason
    job_folder   = os.path.join( results_folder, f"{user_jobname}___ref_{task_id}")                 
    
    try:
        user_jobname, job_folder = get_jobname_and_dir( results_folder, task_id )        
        try:
            shutil.rmtree( job_folder )                             # must remove this 'remember the user_jobname' directory, 
                                                                    # immediately before (re-)using name for
                                                                    # rename of the nnnn_results  directory we get when extracting the zip
            logger.debug( f"'{job_folder}' was removed" )
            
        except OSError as ex:
            logger.warning( f"Error: {ex}. Directory did not exist" )

    except ValueError as ex:
        logger.warning( f'Value error {ex}' )                       # cannot find task_id in the results folder, 
                                                                    # placeholder lost or not written 
                                                                    
    logger.debug( f"task ID: {task_id}  User_jobname {user_jobname}  dir={job_folder}" )
        
        
    # RENAMES:
  
    # folder name
    current_path = os.path.join( results_folder, task_id + "_results")
    logger.info(f"renaming FOLDER from\n{current_path}  to\n{job_folder}")
    os.rename( current_path, job_folder)
    
    # original.png
    orig_path     = os.path.join( job_folder, "original.png")
    new_orig_file = f"original___{user_jobname}___ref_{task_id}.png"
    new_orig_path = os.path.join( job_folder, new_orig_file)
    logger.info(f"renaming ORIGINAL from\n{orig_path}  to\n{new_orig_path}")
    os.rename( orig_path, new_orig_path)
       
    # Pristina-Result.png
    result_path     = os.path.join( job_folder, "Pristina-Result.png")
    new_result_file = f"Pristina___{user_jobname}___ref_{task_id}.png"
    new_result_path = os.path.join( job_folder, new_result_file)
    logger.info(f"renaming PrRESULT from\n{result_path}  to\n{new_result_path}")
    os.rename( result_path, new_result_path)    
    
    



@retrex()
def download_and_unzip( storage_path,  results_folder,  this_thread_id): 
    
    logger.info( f'\n\n\t***** STARTING DOWNLOAD AND UNZIP *****\n\n')
   
    if IN_BLENDER  and  results_folder.startswith("//"):        
        results_folder = bpy.path.abspath( results_folder)

    logger.info(f"Starting download...  \n\tstorage_path={storage_path}\n\tresults_folder={results_folder}")
    sb_client   = SessionManager.get_client( this_thread_id)  

    # temp try and except while alternating between public access and private/RLS for 'pristina_results' bucket -- todo set RLS
    try:
        data    = sb_client.storage.from_("pristina_results"       ).download( storage_path)  # when RLS set -- returns bytes object
    except:
        data    = sb_client.storage.from_("public/pristina_results").download( storage_path)  # when bucket set to PUBLIC
    
    # logger.critical(f'\n\n\n\t***** data = {data[:180]} \n\n\n')    # todo temp remove or comment out (get first 180 bytes)

    temp_zip_path = os.path.join( results_folder, "temp.zip")    
    logger.info(f'temp_zip_path={temp_zip_path}')
    
    with open(temp_zip_path, 'wb') as file:         # see note end of script, best for large downloads, file-as-you-go approach
        file.write(data)                            # todo consider this upgrade on next plugin version
    
    logger.info("Download completed successfully.")
    
    logger.info(f"Starting extraction...  to {results_folder}")
    
    task_id = get_task_id( temp_zip_path)
                            
    logger.info(f'extracting zip >>>>>>>>>>>>\n\tfrom temp_zip_path={temp_zip_path}\n\tto results_folder={results_folder}\n')
    
    with zipfile.ZipFile(temp_zip_path, 'r') as zip_ref:
        zip_ref.extractall( results_folder)
        
    do_renames(  results_folder, task_id )                
    
    logger.info(f"Extraction completed successfully. tzp={temp_zip_path}")
    
    if os.path.exists( temp_zip_path):
        os.remove(temp_zip_path)  
    logger.info("Temporary files cleaned up.")
        
    
    # ------------------------------------------------------------   DOWNLOADED OK, SET STATUS TO 'FETCHED'
    
    result       = set_status_to_fetched(  this_thread_id, task_id)  
    
    if result['error']:
        logger.warning(f"{result['message']}")
        pass                                        # log the update error, but don't stop the show...  non critical    
        
    result = LogUploader.on_good_success(task_id)   # currently ignore result, not critical (mainly used for testing)
        
    return  {'error':  False}                       # retrex compatible (dictionary return)




'''
# latest supabase api recommendation
# https://supabase.com/docs/reference/python/storage-from-upload

with open(destination, 'wb+') as f:
  res = supabase.storage.from_('bucket_name').download(source)
  f.write(res)
'''




@retrex()
def  set_status_to_fetched(  session_name, task_id):    
    logger.info( f'***** SETTING TASK {task_id} STATUS TO FETCHED *****\n\n')
    sb_client       = SessionManager.get_client( session_name)      
    update_values   = { "status": 'FETCHED'  }
    # logger.info(f'about to update status for {task_id} with {update_values}')
    result          = sb_client.table( 'task_queue').update( update_values).eq( "id", task_id).execute()
    # logger.info(f'>>>>>>>>>>>>>>>>>>>>>>>>>> result = {result}')
    return {'error': False}   # no-error return  (if there was an exception then retrex will return instead, with  'message')


# -----------------------------------------  MD5 ---------------  used to spot duplicate jobs by task processor/supabase sql function

import hashlib

def  get_md5( file_path):
    hash_md5 = hashlib.md5()
    try:
        with open( file_path, "rb" ) as f:
            for chunk in iter( lambda: f.read( 4096 ), b"" ):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()      # return is always 32 hex digit string
        
    except FileNotFoundError:
        msg = f'file not found  { file_path}'
        logger.error( msg)
        raise RuntimeError( msg)
        
    except Exception as e:
        msg = f'{e}'
        logger.exception( msg)
        raise RuntimeError( msg)

