import bpy
import os

class ImageResizer:
    def __init__(self, input_image, output_name, output_path, reference_strength):
        self.input_image = input_image
        self.output_name = output_name
        self.output_path = bpy.path.abspath(output_path)
        self.reference_strength = reference_strength

    def resize_and_save(self):
        img = self.input_image
        if img is None:
            raise ValueError("Image could not be loaded.")

        pixels = list(img.pixels[:])
        img_data = [(pixels[i], pixels[i+1], pixels[i+2], pixels[i+3] if img.channels == 4 else 1.0) for i in range(0, len(pixels), img.channels)]
        avg_color = [sum(c[i] for c in img_data) / len(img_data) for i in range(3)]
        
        new_width, new_height = self.calculate_new_dimensions(img.size[0], img.size[1])
        resized_img = bpy.data.images.new("temp_img", width=new_width, height=new_height, alpha=False)
        resized_pixels = [0.0] * (new_width * new_height * 4)

        for y in range(new_height):
            for x in range(new_width):
                src_index = (int(y * img.size[1] / new_height) * img.size[0] + int(x * img.size[0] / new_width)) * img.channels
                dest_index = (y * new_width + x) * 4
                for c in range(3):
                    resized_pixels[dest_index + c] = pixels[src_index + c] * (pixels[src_index + 3] if img.channels == 4 else 1.0) + avg_color[c] * (1 - (pixels[src_index + 3] if img.channels == 4 else 1.0))
                resized_pixels[dest_index + 3] = 1.0

        resized_img.pixels = resized_pixels
        final_pixels = [avg_color[i % 3] if i % 4 != 3 else 1.0 for i in range(1024 * 1024 * 4)]
        offset_x, offset_y = (1024 - new_width) // 2, (1024 - new_height) // 2

        for y in range(new_height):
            for x in range(new_width):
                src_index, dest_index = (y * new_width + x) * 4, ((y + offset_y) * 1024 + (x + offset_x)) * 4
                final_pixels[dest_index:dest_index+4] = resized_pixels[src_index:src_index+4]

        temp_img = bpy.data.images.new("temp_img_for_save", width=1024, height=1024, alpha=False)
        temp_img.pixels = final_pixels
        output_file_path = os.path.join(self.output_path, self.output_name)
        temp_img.filepath_raw = output_file_path
        temp_img.file_format = 'JPEG'
        temp_img.save()
        bpy.data.images.remove(temp_img)
        return output_file_path

    def calculate_new_dimensions(self, width, height):
        if width == 0 or height == 0:
            raise ValueError("Width or height of the image cannot be zero.")
        aspect_ratio = width / height
        return (1024, int(1024 / aspect_ratio)) if aspect_ratio > 1 else (int(1024 * aspect_ratio), 1024)
