# install_completed____popup_dialog.py



import bpy  
import sys
import os

# when install successfully completes, this popup is called to notify user
# It explains that Blender must be rebooted, and admin or root permissions -not- now needed
# then: press OK or Cancel to quit Blender



launch_popup     = False                    # flag has 'one shot' use -- when True it is acted on once, 
                                            # and immediately set back False to stop repeated triggering of popup
                                            
                                                           # get visibility for the package and modules --- append sys path
plugin_dir      = os.path.dirname(__file__)
if plugin_dir not in sys.path:  sys.path.append( plugin_dir)


from logger_module    import   Logger       # setup logger
logger          = Logger.get_logger()       
                              
                                            
     
# ----------------------------------------------------------------------------------    
 
def launch_install_completed_popup():
    logger.info(f'\n\n\t*****popup flag={launch_popup} *****\n\n')
    
    if launch_popup:
        try:
            bpy.utils.register_class(ConfirmShutdownOperator2)
        except Exception as ex:
            logger.warning(f'could not register ConfirmShutdownOperator2')                                    
        
        logger.info("popup flag set to True.")
        bpy.app.timers.register( trigger_confirmation_dialog, first_interval=0.1)




class ConfirmShutdownOperator2(bpy.types.Operator):      
    
    bl_idname  = "wm.confirm_shutdown_dialog2"
    bl_label   = "Install complete:  Restart Blender"    # title of the popup dialog
    bl_options = {'REGISTER', 'UNDO'}
    
    def execute(self, context):
        logger.info("ConfirmShutdownOperator2.execute() called, cleaning up and shutting down Blender...")        
        try:
            bpy.utils.unregister_class( ConfirmShutdownOperator2)
            logger.info( f'unregistered ConfirmShutdownOperator2')

        except Exception as ex:
            logger.error( f'could not unregister ConfirmShutdownOperator2  {ex}')
            
        bpy.ops.wm.quit_blender()                       # Quit Blender
        return {'FINISHED'}
        
        ##  see end of script for error message seen from this method
        
        
        

    def cancel(self, context):                          # Treat cancel button press same as the OK by calling execute        
        logger.info("ConfirmShutdownOperator2.cancel() called, treating cancel as OK...")
        #return self.execute(context)           # causes error as return value (if given) should be None or {'CANCELLED'}
        self.execute(context)                   # just want this functionality, don't need return value
            
    
    def draw(self, context):
        layout = self.layout
        layout.label(text="")  
        layout.label(text="Installation has completed successfully.")   # notification text shown in box
        layout.label(text="Blender must now be restarted")    
        layout.label(text="** Admin or Root permissions are -not- now needed **")      
        layout.label(text="")  
        layout.label(text="Press 'OK'  (or 'Cancel')  to quit Blender.")    
    
    
    def invoke(self, context, event):
        logger.info("ConfirmShutdownOperator2.invoke() called, showing confirmation dialog...")               
        return context.window_manager.invoke_props_dialog(self)  # Open a modal dialog with only OK and Cancel buttons




def trigger_confirmation_dialog():  # Function to trigger the modal dialog safely in the Preferences window context
    global launch_popup
    
    if launch_popup:
        launch_popup = False        # Reset the flag immediately to avoid repeated triggering
        
        logger.info("Diagnostic: Invoking custom confirmation dialog in Preferences context...")
        
        preferences_area = None     # Make sure the dialog is associated with the Preferences window context

        # Find the Preferences window context
        for window in bpy.context.window_manager.windows:
            for area in window.screen.areas:
                if area.type == 'PREFERENCES':
                    preferences_area = area
                    break

        if preferences_area:
            with bpy.context.temp_override(window=window, area=preferences_area):
                bpy.ops.wm.confirm_shutdown_dialog2('INVOKE_DEFAULT')
                
   
                
        else:
            logger.info("Diagnostic: Preferences area not found, cannot trigger confirmation dialog.")
    
    return None                     # Stops the timer from repeating





def register():
    try:    
        bpy.utils.register_class( ConfirmShutdownOperator2)     # may already have been registered        
    except Exception:
        pass               
    launch_install_completed_popup()
    



'''
14:17:47 - INFO     - install_completed____popup_dialog - line  66 in cancel - ConfirmShutdownOperator2.cancel() called, treating cancel as OK...

14:17:47 - INFO     - install_completed____popup_dialog - line  52 in execute - ConfirmShutdownOperator2.execute() called, cleaning up and shutting down Blender...

14:17:47 - INFO     - install_completed____popup_dialog - line  55 in execute - registered ConfirmShutdownOperator2
RuntimeError: expected class (�F��☺, function cancel to return None, not set
Preferences window closed.


'''