# object_analyzer.py


# from object_analyzer  import ObjectAnalyzer


import bpy

from logger_module                  import   Logger

logger       = Logger.get_logger()      ####################  global, is set up on load 




class ObjectAnalyzer:
    def __init__(self, object_names):
        self.object_names = object_names

    def mist_setup(self, start, depth):
        # Ensure compositor nodes are enabled
        bpy.context.scene.use_nodes = True

        # Access and configure world mist settings
        logger.info( "changing MIST SETTINGS NOW")
        world_settings          = bpy.context.scene.world
        mist_settings           = world_settings.mist_settings
        mist_settings.use_mist  = True
        mist_settings.intensity = 1
        mist_settings.falloff   = 'LINEAR'
        mist_settings.start     = start #depth
        mist_settings.depth     = depth #start
        
        bpy.context.scene.render.image_settings.file_format = 'PNG'
        bpy.context.scene.render.image_settings.color_mode  = 'RGB'
        bpy.context.scene.render.image_settings.compression = 30

        # Enable Mist pass for the current view layer
        bpy.context.scene.view_layers[bpy.context.view_layer.name].use_pass_mist = True

    def find_extreme_vertices(self, obj):
            if obj and obj.type == 'MESH':  # Ensure object is valid and is a mesh
                camera_loc         = bpy.context.scene.camera.location
                obj_vertices_world = [obj.matrix_world @ v.co for v in obj.data.vertices]

                # Calculate vertex distances from camera and find the extremes
                distances          = [(v - camera_loc).length for v in obj_vertices_world]
                furthest_vertex    = obj_vertices_world[distances.index(max(distances))]
                closest_vertex     = obj_vertices_world[distances.index(min(distances))]

                return furthest_vertex, closest_vertex
            else:
                # Return None or some default value if the object is not valid
                return None, None
        
    def analyze_objects(self):
        closest_distance, furthest_distance = float('inf'), 0.0

        for object_name in self.object_names:
            obj = bpy.data.objects.get(object_name)
            if obj and obj.type == 'MESH':
                furthest_vertex, closest_vertex = self.find_extreme_vertices(obj)

                camera_loc        = bpy.context.scene.camera.location
                furthest_distance = max(furthest_distance, (furthest_vertex - camera_loc).length)
                closest_distance  = min(closest_distance,  (closest_vertex  - camera_loc).length)

        # Print and apply mist settings based on calculated distances
        if closest_distance < float('inf') and furthest_distance > 0.0:
            logger.info( f"Closest Distance to Camera: {closest_distance}")
            logger.info( f"Furthest Distance to Camera: {furthest_distance}")
            depth = furthest_distance - closest_distance
            logger.info( f"Depth: {depth}")

            self.mist_setup(closest_distance, depth)
        else:
            logger.info( "No valid mesh objects found.")
            self.mist_setup(0, 25)