# ot_add_move_remove_items.py

# from ot_add_move_remove_items  import   PRISTINA_OT_AddItem,  PRISTINA_OT_MoveItem,  PRISTINA_OT_RemoveItem


import bpy
from logger_module import Logger


logger       = Logger.get_logger() 






class PRISTINA_OT_AddItem( bpy.types.Operator):
    bl_idname       = "scene.pristina_add_item"
    bl_label        = "Add Item"
    bl_description  = "Add item"
    
    def execute(self, context):
        scene           = context.scene
        pristina_props  = scene.pristina_properties
        new_item        = pristina_props.pristina_items.add()

        selected_object = bpy.context.active_object
        if selected_object:
            new_item.name = selected_object.name
            temp_props    = {}
            custom_props  = [k for k in selected_object.keys() if k != '_RNA_UI']
            for prop_name in custom_props:
                temp_props[prop_name] = selected_object.get(prop_name)

            new_item.object_input = selected_object

            for prop_name, prop_value in temp_props.items():
                if hasattr(new_item, prop_name):
                    setattr(new_item, prop_name, prop_value)
                    logger.info(f"Applied property: {prop_name} with value: {prop_value} to {new_item.name}")

        pristina_props.active_item_index = len(pristina_props.pristina_items) - 1
        return {'FINISHED'}
        
        
        
        
        

class PRISTINA_OT_MoveItem( bpy.types.Operator):
    bl_idname       = "pristina.move_item"
    bl_label        = "Move Item"
    bl_description  = "Move Item"  # tooltip text is fixed here, cannot alter according to direction given in call
    
    direction: bpy.props.EnumProperty( items = [('UP', 'Up', 'Move Up'), ('DOWN', 'Down', 'Move Down')]  )

    def execute( self, context):
        scene          = context.scene
        pristina_props = scene.pristina_properties
        active_index   = pristina_props.active_item_index
        new_index      = active_index - 1 if self.direction == 'UP' else active_index + 1

        if 0 <= new_index < len( pristina_props.pristina_items):
            pristina_props.pristina_items.move( active_index, new_index)
            pristina_props.active_item_index = new_index
        return {'FINISHED'}
        
        
        


class PRISTINA_OT_RemoveItem( bpy.types.Operator):
    bl_idname       = "pristina.remove_item"
    bl_label        = "Remove Item"
    bl_description  = "Remove item"

    def execute(self, context):
        scene = context.scene
        pristina_props = scene.pristina_properties
        if pristina_props.pristina_items and pristina_props.active_item_index < len(pristina_props.pristina_items):
            pristina_props.pristina_items.remove(pristina_props.active_item_index)
            pristina_props.active_item_index = min(pristina_props.active_item_index, len(pristina_props.pristina_items) - 1)
        return {'FINISHED'}
        
        
                