# ot_login.py

# from ot_login import   PRISTINA_OT_Login


import bpy
from   sessions import SessionManager
import popup_for_low_credits
import popup_for_bad_login
from   logger_module        import Logger

logger       = Logger.get_logger()      ####################  global, is set up on load 




def fetch_credits( sb_client, user_id):
    try:
        response = sb_client.table("profiles").select("credits").eq("id", user_id).execute()
        return response.data[0]["credits"]
    except:
        return None
    
    


class PRISTINA_OT_Login( bpy.types.Operator):
    
    bl_idname      = "pristina.login"
    bl_label       = "Login"
    bl_description = "Login"


    def execute(self, context):
        
        pristina_props  = context.scene.pristina_properties
        
        SessionManager.setup( pristina_props.email,   pristina_props.password )     # sets these props as class vars and 
                                                                                    # creates dedicated thread to check for session expiry
        
        session_name    = SessionManager.get_thread_id_with_name( 'Main' )          # appends the thread_id for easy recognition  eg '13928--Main'

        try:
            sb_client       = SessionManager.get_client( session_name)  
            if sb_client is None:
                raise RuntimeError(f'sign-in failed for {pristina_props.email} with px={pristina_props.password}')
            
            response        = sb_client.auth.get_user()
            user            = response.user            
                                                        # logger.critical(f'\n\n\t**** user = \n{ user}\n\n')        
            pristina_props.auth_status = True
            pristina_props.user_id     = user.id
            credits                    = fetch_credits( sb_client, user.id)
            pristina_props.credits     = credits if credits is not None else 0            
            popup_for_low_credits.check_credits( pristina_props.credits)   # check if less than threshold (eg 100) and prompt to top up            
            self.report({'INFO'}, f"Logged in as {user.email}. Credits: {pristina_props.credits}")  
            
        except Exception as ex:
            popup_for_bad_login.do_popup()              # bad login -- ask user to supply valid credentials and try again
            logger.exception( f'\n\n\t***** Login failed for {pristina_props.email} *****\n\n\t{ex}\n\n')
            pristina_props.auth_status = False
            pristina_props.user_id     = ""
            pristina_props.credits     = 0              # todo make sure on server not to accept job if not enough credits (ie if this code hacked to 'adjust credits')
            # self.report({'ERROR'}, "Failed to log in.")   

        finally:
            SessionManager.logout( session_name)        # close session at earliest opportunity, reopen when required (next job)
              
            
        for area in context.screen.areas:
            if area.type == 'PROPERTIES':
                area.tag_redraw()                       # make sure the ui is updated   
            
        return {'FINISHED'}
        
        
    