# ot_sundry.py

# from ot_sundry import    PRISTINA_OT_OpenLink
# from ot_sundry import    PRISTINA_OT_SortByDistance
# from ot_sundry import    ERROR_OT_popup_message


import bpy




class PRISTINA_OT_SyncResults( bpy.types.Operator):
    bl_idname      = "pristina.sync_results"
    bl_label       = "Sync"
    bl_description = "Sync: download results from any interrupted job(s)"

    def execute( self, context):        
        local_path = bpy.context.scene.pristina_properties.file_output         
        if os.path.isdir( local_path ):            
            start_fetch_in_background_thread(   local_path )      
                                            # see sync.py module        
                                            # this method downloads any results not yet fetched, 
                                            # eg when plugin closed before job(s) finished on server
                                            
                                            # todo -- consider persisting jobnames at start of job 
                                            # so can 'marry up' jobname with late downloads if needed
                                            # else recovered jobs will probably lose their given name or all share 
                                            # last name kept by Blender   

                                            # overall reason is that jobnames are likely confidential, and 
                                            # so policy is not to upload and store on supabase, just keep in users's own machine
                                            # easy to do during one plugin session, blender var is set to name, then used to rename
                                            # results when downloaded. However, in a new plugin session, the previous jobname(s) will be lost
                                            # to do submit 2 jobs in quick succession and check they are correctly renamed when results downloaded

                                            
        else:
            logger.error(f'local path is not valid -- {local_path}')                    
        return {'FINISHED'}
        
        
        
   

class PRISTINA_OT_OpenLink( bpy.types.Operator):
    bl_idname      = "pristina.open_link"
    bl_label       = "Open Pristina"
    bl_description = "Visit Pristina.ai website to purchase more credits"

    def execute( self, context):
        import webbrowser
        webbrowser.open("https://www.pristina.ai/pricing/")
        return {'FINISHED'}
        
        
        

class PRISTINA_OT_SortByDistance( bpy.types.Operator):
    """Sort Items by Distance from Camera"""
    bl_idname      = "pristina.sort_by_distance"
    bl_label       = "Sort Items by Distance from Camera"
    bl_description = "Auto arrange items based on distance from camera, closest at the top"

    def execute( self, context):
        pristina_props  = context.scene.pristina_properties
        camera_location = context.scene.camera.location

        # Create a list of tuples, each containing an item's index and its distance to the camera
        items_with_distances = []
        for index, item in enumerate(pristina_props.pristina_items):
            if item.object_input:  # Ensure the item has an associated object
                obj_location = item.object_input.location  # Get the object's origin location
                distance     = (camera_location - obj_location).length  # Calculate the distance
                items_with_distances.append((index, distance))

        # Sort the list of tuples by the distance (second element of the tuple)
        items_with_distances.sort( key=lambda x: x[1])

        # To reorder the collection we need to reposition items using their current and new indices
        # We do this by calculating new indices from the sorted list
        new_order = [item[0] for item in items_with_distances]

        # Move items to their new positions using a stable method
        already_moved = set()
        for target_index, original_index in enumerate(new_order):
            if target_index != original_index and original_index not in already_moved:
                pristina_props.pristina_items.move(original_index, target_index)
                already_moved.add(target_index)

        return {'FINISHED'}



class LoadImageFromFileOperator(bpy.types.Operator):
    bl_idname = "image.load_image_from_file"
    bl_label = "Load Image from File"

    def execute(self, context):
        scene = context.scene
        if scene.image_filepath:
            try:
                img = bpy.data.images.load(scene.image_filepath)
                scene.image = img
                update_image(scene, context)
                self.report({'INFO'}, "Image loaded successfully")
            except Exception as e:
                self.report({'ERROR'}, f"Failed to load image: {str(e)}")
                return {'CANCELLED'}
        else:
            self.report({'ERROR'}, "No image file path provided")
            return {'CANCELLED'}
        return {'FINISHED'}

def update_image(self, context):
    if self.image:
        context.scene.image_path = self.image.filepath_raw
    else:
        context.scene.image_path = ""



class ERROR_OT_popup_message( bpy.types.Operator):
    """Display an error message in a popup"""
    bl_idname = "error.popup_message"
    bl_label  = "Error Message"
    
    message: bpy.props.StringProperty()  # Property to hold the message

    def execute( self, context):
        self.report({'ERROR'}, self.message)
        return {'FINISHED'}
    
    def invoke( self, context, event):
        wm    = context.window_manager
        return wm.invoke_props_dialog(self)  # This creates the popup dialog

    def draw( self, context):
        self.layout.label(text=self.message)

  
  
  