# pip_installs.py


#import bpy
import sys
import os
import shutil
import subprocess
import ensurepip
import importlib
import time


from logger_module    import   Logger

logger       = Logger.get_logger()      

         
            
            

    

def install_package( package_name, version=None):
    try:
        if version:
            package_spec = f"{package_name}=={version}"
        else:
            package_spec = f"{package_name}"
        logger.info(f"Attempting to install package: {package_spec}  using:\n\t{sys.executable}\n")
        
        subprocess.check_call([ sys.executable,     "-m", 
                                                    "pip", 
                                                    "install", 
                                                    "--ignore-installed",
                                                    package_spec])
        
        logger.info(f"Successfully installed package: {package_spec}")

    except subprocess.CalledProcessError as ex:
        logger.error(f"Failed to install package {package_name}=={version}. Error: {ex}")

    except Exception as ex:
        logger.exception(f"An unexpected error occurred while installing package {package_name}=={version}. Error: {ex}")    
    

            
            
            
def XXXensure_dependencies_installed():
    

            
    packages_to_install = [     ('supabase', None), # package =-- just get latest 
    #packages_to_install = [     ('supabase', '2.10.0'), # package, version (specify known-good version, 
                                                        # don't leave it up to pip to choose)
                                                        # add others here if needed                                    
    ]          
    
    # expect: to force upgrade to new version specified above,
    # open explorer as admin to see (for example)
    # C:\Program Files\Blender Foundation\Blender 4.1\4.1\python\lib\site-packages
    # and delete or rename the 2 supabase files (itself and dist-info) eg   to Xsupabase  etc
    # then the code below should not find it and it should do new install with new version
    # check the pristina.log file to see it has actually done this
    
    logger.info(f'ensuring pip')
    ensurepip.bootstrap()
        
    try:
        
        for package, version in packages_to_install:    # iterate over list of tuples
            
            try:
                _ = importlib.import_module( package)   # Dynamically import the package
                
            except Exception as ex:         
                logger.info( f"\n\nImporting {package} failed \n{ex}\nso installing version {version}\n\n")
                install_package( package, version)
                time.sleep( 5 )                         # Add a delay to allow the installation to complete
                #time.sleep( 10 )                         # Add a delay to allow the installation to complete
                importlib.invalidate_caches()           # Invalidate the import caches

        
            try:
                module = importlib.import_module( package)
                logger.info(f"{package} version: {     getattr(module, '__version__', 'Unknown')} installed ok")
                logger.info(f"{package} installed at: {getattr(module, '__file__',    'Unknown location')}")  
                
            except ImportError as ex:
                logger.critical( f"{package} installation failed  {ex}.")    
                raise                

                                                        # Log the version of Supabase explicitly at the end
            try:
                supabase = importlib.import_module("supabase")
                logger.info(f"Supabase version in use: {getattr(supabase, '__version__', 'Unknown')}")
            except Exception as ex:
                logger.warning(f"Could not determine Supabase version at the end. Exception: {ex}")                
                                
                
        logger.info( f'\n\n\t***** Dependencies verified ok *****\n\n')
        
                
    except Exception as ex:
        logger.critical( f'\n\n\n**** COULD NOT ENSURE DEPENDENCIES INSTALLED *****\n\nException {ex}\n\n\n')
        raise
            

def ensure_dependencies_installed():
    packages_to_install = [ ('supabase', '2.10.0') ]  # Add other packages if necessary
    
    logger.info(f'ensuring pip')
    ensurepip.bootstrap()

    try:
        for package, version in packages_to_install:
            try:
                #_ = importlib.import_module(package)
                import supabase
            except Exception as ex:
                logger.info(f"Importing {package} failed. Installing version {version}.")
                install_package(package, version)
                logger.info("Waiting for installation to stabilize...")
                time.sleep( 5)  # wait time to ensure installation completes
                importlib.invalidate_caches()  # Invalidate import caches

            '''
            # Retry import after delay
            retries = 10
            for retry in range(retries):  # Attempt 3 retries if import fails
                time.sleep(2)
                logger.info(f'checking supabase retry = {retry}')
                try:
                    import supabase
                    #module = importlib.import_module(package)
                    #logger.info(f"{package} version: {getattr(module, '__version__', 'Unknown')} installed ok")
                    #logger.info(f"{package} installed at: {getattr(module, '__file__', 'Unknown location')}")
                    break
                except ImportError as ex:
                    logger.warning(f"Retry {retry + 1}: Import failed for {package}. Exception: {ex}")
                    if retry == retries-1:  # Final retry failed
                        logger.critical(f"Failed to import {package} after installation. Raising exception.")
                        raise
            '''
                
        #logger.info(f"\n\n\t***** Dependencies verified ok *****\n\n")
        logger.info(f"\n\n\t***** Dependency verification skipped *****\n\n")

    except Exception as ex:
        logger.critical(f"\n\n\n**** COULD NOT ENSURE DEPENDENCIES INSTALLED *****\n\nException: {ex}\n\n\n")
        raise
