# popup_for_job_fail.py


import bpy
from logger_module        import Logger

logger       = Logger.get_logger()      ####################  global, is set up on load 




def notify(task_id):
    ''' Trigger the modal popup to notify the user of task failure'''
    bpy.ops.wm.task_failed_popup('INVOKE_DEFAULT', task_id= str(task_id))    # call to op, expects string, not int


class TaskFailedPopupOperator(bpy.types.Operator):
    """Popup to notify user that a task has failed"""
    bl_idname = "wm.task_failed_popup"                  # op name:   task_failed_popup
    bl_label = "Task Failure"

    task_id: bpy.props.StringProperty()                 # Holds the task ID passed to notify()

    def execute(self, context):       
        logger.info(f"Notification popup: Task {self.task_id} failed, acknowledged")  
        return {'FINISHED'}

    def draw(self, context):
        layout = self.layout
        layout.label(text=f"")    # popup mesg
        layout.label(text=f"Unfortunately, task {self.task_id} failed.")    # popup mesg
        layout.label(text=f"")    # popup mesg
        layout.label(text=f"You have -not- been charged for this job.")     # popup mesg

    def invoke(self, context, event):        
        return context.window_manager.invoke_props_dialog(self)             # Invoke the dialog
    


def register():
    bpy.utils.register_class(TaskFailedPopupOperator)
    # Test the notification immediately after registration
    # notify("12345")


def unregister():
    bpy.utils.unregister_class(TaskFailedPopupOperator)

