# pristina_item_and_properties.py


# from pristina_item_and_properties   import     PristinaItem, PristinaProperties
 

import bpy
from nested_ui_panels   import PRISTINA_PT_MainPanel


class PristinaItem(bpy.types.PropertyGroup):
    
    skip_update:        bpy.props.BoolProperty(   
                                            default= False)
    
    export_mask:        bpy.props.BoolProperty(   
                                            name        = "Export Mask", 
                                            description = "Export a separate transparent PNG mask of this item",
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    object_input:       bpy.props.PointerProperty(
                                            type        = bpy.types.Object, 
                                            name        = "Object", 
                                            description = "Pristina item will apply to this object",
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    prompt:             bpy.props.StringProperty( 
                                            name        = "Prompt", 
                                            description = "Description of your object, include important details",
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    negative_prompt:    bpy.props.StringProperty(
                                            name        = "Negative Prompt", 
                                            description = "Undesirable elements or characteristics to "
                                                          "exclude from the generated image",
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    denoise_strength:   bpy.props.FloatProperty(
                                            name        = "AI Strength", 
                                            default     = 0.5, 
                                            min         = 0.0, 
                                            max         = 1.0, 
                                            description = "Strength of the overall AI intensity, "
                                                          "higher values steer further away from the original",
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    geometry_strength:  bpy.props.FloatProperty(
                                            name        = "Geometry Strength",    
                                            default     = 0.9, 
                                            min         = 0.0, 
                                            max         = 1.0, 
                                            description = "Strength of Geometry influence, higher values "
                                                          "are more accurate to the base model",
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    sd_seed:            bpy.props.FloatProperty(       
                                            name        = "sd Seed", 
                                            default     = -1, 
                                            min         = -1, 
                                            max         = 999999, 
                                            description = "Seed number, can be used to produce more consistent results."
                                                          "Values 0 or below will produce a random seed. Each render will",
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    quality:            bpy.props.EnumProperty(
                                            name        = "Quality",
                                            items       =[
                                                ('Fast',  'Fast',  ''),
                                                ('Good',  'Good',  ''),
                                                ('Best',  'Best',  ''),
                                                ('Ultra', 'Ultra', '')
                                            ],
                                            description = "Increase or decrease the object detail, "
                                                          "low quality settings render quicker",
                                            default     = 'Good',
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    presets:            bpy.props.EnumProperty(
                                            name        = "Presets",
                                            items       = [
                                                ('Human', 'Human / Person', 'Human / Person'),
                                                ('Vegetation', 'Vegetation', 'Vegetation'),
                                                ('Product', 'Product', 'Product'),
                                                ('Furniture', 'Furniture', 'Furniture'),
                                                ('Water', 'Water / Lake / Sea', 'Water / Lake / Sea'),
                                                ('Sky', 'Sky / Clouds', 'Sky / Clouds'),
                                                ('Artwork', 'Artwork / Illustration', 'Artwork / Illustration'),
                                                ('General', 'General', 'General')
                                            ],
                                            description = "Presets: Choose from the dropdown list",
                                            default     = 'General',
                                            update      = lambda self, context: self.update_custom_properties(context))
                                            
    restore_faces:      bpy.props.BoolProperty(
                                            name        = "Restore Faces", 
                                            default     = False, 
                                            description = "Restore Human Faces: ensures faces don't appear distorted or morphed",
                                            update      = lambda self, context: self.update_custom_properties(context))

    use_reference_image: bpy.props.BoolProperty(
                                            name        = "Use Reference Image",
                                            default     = False,
                                            description = "Use a reference image to influence the look of this item",
                                            update      = lambda self, context: self.update_custom_properties(context))

    reference_image_path: bpy.props.StringProperty(
                                            name        = "Reference Image Path",
                                            subtype     = 'FILE_PATH',
                                            description = "Reference Image path",
                                            update      = lambda self, context: self.update_custom_properties(context))

    reference_strength:  bpy.props.FloatProperty(
                                            name        = "Reference Strength",    
                                            default     = 0.9, 
                                            min         = 0.0, 
                                            max         = 1.0, 
                                            description = "Higher values increase influence of reference image",
                                            update      = lambda self, context: self.update_custom_properties(context))


    
    def update_custom_properties(self, context):     # Custom update function to apply properties to the object's custom properties
        
        if self.skip_update or not self.object_input:            
            return                                              # Skip the update if flagged or object_input is not set
            
        obj                         = self.object_input
        obj["export_mask"]          = self.export_mask
        obj["prompt"]               = self.prompt
        obj["negative_prompt"]      = self.negative_prompt
        obj["denoise_strength"]     = self.denoise_strength
        obj["geometry_strength"]    = self.geometry_strength
        obj["sd_seed"]              = self.sd_seed
        obj["quality"]              = self.quality
        obj["presets"]              = self.presets
        obj["restore_faces"]        = self.restore_faces
        obj["use_reference_image"]  = self.use_reference_image
        obj["reference_image_path"] = self.reference_image_path
        obj["reference_strength"]   = self.reference_strength
        
        

DASHES = '--------'     # used to show password has been transferred into private/internal variable
                        # prevents it being copied and pasted from ui display field



class PristinaProperties(bpy.types.PropertyGroup):
    
    
    pristina_items:     bpy.props.CollectionProperty(
                                            type        = PristinaItem)
                                            
                                            
    active_item_index:  bpy.props.IntProperty(
                                            default     = 0, 
                                            min         = 0)
                                            
                                            
    file_name:          bpy.props.StringProperty(       # XXXXXXXXXXX  no longer shown in UI   XXXXXXXXXXXXX
                                            name        = "File Name", 
                                            default     = "render_output", 
                                            description = "Render Output -- give a name for the result file\n"
                                                          "(without extension), eg  render_output",
                                            maxlen      = 255)
                                            
                                            
    job_name:           bpy.props.StringProperty(       # will be joined with task_id, to make human readable
                                            name        = "Job Name", 
                                            default     = "", 
                                            description = "Give a name for this job. This will be used to name\n"
                                                          "the local job output folder. This name is not uploaded"
                                                          "and remains private to your computer.",
                                            update      = lambda  self, context: PRISTINA_PT_MainPanel.redraw_panel(),
                                            maxlen      = 230)          # leave room to concat the task_id
                                            
    
    file_output:        bpy.props.StringProperty(  
                                            name        = "Render Output", 
                                            description = "Output folder to save the resulting processed render",
                                            default     = "", 
                                            subtype     = 'DIR_PATH')
                                            
                                            
    email:              bpy.props.StringProperty(
                                            name        = "Email", 
                                            default     = "", 
                                            maxlen      = 64, 
                                            description = "Enter your email address")
                                            
                                            
    # ----------------------------------------------------------------------------------------------- PASSWORD
                                
    # have 2 vars for password, one for input, then immediately copy to private var
    # and set the input var to blank  -- this to avoid copying from screen
    # the screen does show blanked out characters, but copy and paste easily defeats this
    # hence moving password to private var
                                
    
    password:           bpy.props.StringProperty(                
                                            name        = "Password", 
                                            default     = "tba", 
                                            maxlen      = 64, 
                                            subtype     = 'PASSWORD')
                                            

    password_input:     bpy.props.StringProperty(
                                            name        = "Password Input", 
                                            default     = DASHES, 
                                            maxlen      = 64, 
                                            description = "Enter your password", 
                                                          # update copies 'password_input' to the private variable, then sets input blank
                                            update      = lambda self, context: self.update_password_input(context), 
                                            subtype     = 'PASSWORD')


    # -----------------------------------------------------------------------------------------------
 

 
    auth_status:        bpy.props.BoolProperty(
                                            name        = "Authentication Status", 
                                            default     = False)                    
    
    user_id:            bpy.props.StringProperty(
                                            name        = "User ID", 
                                            default     = "", 
                                            description = "User ID")
    
    credits:            bpy.props.IntProperty(
                                            name        = "Credits", 
                                            default     = 0, 
                                            description = "User Credits")
                                            
    
    jobs_in_progress:   bpy.props.IntProperty(
                                            name        = "Jobs in progress", 
                                            default     = 0, 
                                            description = "Total pending and actioned jobs",
                                            update      = lambda  self, context:  PRISTINA_PT_MainPanel.redraw_panel())
        

    globalprompt:       bpy.props.StringProperty(
                                            name        = "GlobalPrompt", 
                                            default     = "", 
                                            maxlen      = 255, 
                                            description = "Description of universal scene aspects, such as weather and lighting")
    
    globalnegprompt:    bpy.props.StringProperty(
                                            name        = "GlobalNegPrompt", 
                                            default     = "", 
                                            maxlen      = 255, 
                                            description = "Undesirable elements or characteristics to exclude from the scene")
    
                                            
                                         
    upscale_result:     bpy.props.EnumProperty(
                                            name        = "Upscale Result",
                                            items       =[
                                                ('None',        'None',        ''),
                                                ('Fast 2X',     'Fast 2X',     ''),
                                                ('Creative 2X', 'Creative 2X', '')
                                            ],
                                            description = "Upscale and enhance quality beyond your render resolution",
                                            default     = 'None',
                                            update      = lambda  self, context: PRISTINA_PT_MainPanel.redraw_panel())
                                            


                                               
   
    
    def update_password_input(self, context):   # Update function copies public password field into private var
                                                # one-shot, does not keep updating itself (recursion stopped with if... )        
        if self.password_input == DASHES:       # in this case, job already done, public var is just dashes
            return
    
        self.password       = self.password_input
        self.password_input = DASHES
