


#import bpy
#from   bpy.app.handlers import persistent        # for background timer for fifo queue checking
import json
import pprint
import shutil
from datetime import datetime, timedelta, timezone
import time
import threading

import ssl
import gc
import sys
import os


#!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# Blender's installation directory
print('**************************************************** now in sessions ***************************')
python_executable = sys.executable
blender_install_dir = os.path.dirname(os.path.dirname(python_executable))
print(f"^^^^^^^^^^^^^^^^^^^^^^^  Blender installation directory: {blender_install_dir}")
# eg  Blender installation directory: C:\Program Files\Blender Foundation\Blender 4.1\4.1\python
# r"C:\Program Files\Blender Foundation\Blender 4.1\4.1\python\lib\site-packages"   # example path to supabase package
sp = os.path.join( blender_install_dir, "lib", "site-packages")
sys.path.append( sp)
print(f"^^^^^^^^^^^^^^^^^^^^^^^  Blender site packages: {sp}\n\nSys path entries:\n")
for item in sys.path:
    print(item)
print('\n\n')    

#!!!!!!!!!!!!!!!!!!!!!!!!!!!


import supabase   as sb_package




from logger_module                  import   Logger
logger       = Logger.get_logger()      ####################  global, is set up on load 


  
SUPABASE_URL = "https://fdqpbqadouajjpqluikg.supabase.co"                           # Pristina database
SUPABASE_KEY = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJzdXBhYmFzZSIsInJlZiI6ImZkcXBicWFkb3VhampwcWx1aWtnIiwicm9sZSI6ImFub24iLCJpYXQiOjE3MTI2MTYxNDQsImV4cCI6MjAyODE5MjE0NH0.iOSR_WF1vuoAd-mBnvQKKgCsty8pFn3utKngyVLzC-w"             # Anon/public key


# ------------------------------------------------------------------------------- Pristina Session object  (private to this module)
class _PrSession:
    
    # see notes in the SessionManager class next below
    
    access_token   = None
    refresh_token  = None
    expires_at_utc = None   
    name           = None
    quick_test     = False
    latest_client  = None
    
    def __init__( self,  email,  password,  name='',  quick_test=False):    
        
        
        
        logger.info(f"\n\n\t***** creating new _PrSession object '{name}'  {email} *****\n\n")   # Session object carries with it the latest client and tokens
        
        self.name            = name
        self.quick_test      = quick_test

        try:                    
            ''' 
            timeouts determine how long the client will wait for a response from the server during API calls  
            before giving up. These settings are crucial for ensuring that the application handles network 
            latency and unresponsive server behaviour gracefully.        
            '''                            
                                                               
            options   = sb_package.ClientOptions(   postgrest_client_timeout = 10,   
                                                    storage_client_timeout   = 10,   
                                                    schema                   = "public")
            
            sb_client = sb_package.create_client( SUPABASE_URL,  SUPABASE_KEY, options)          
                                                                                        # make this as separate retrex method with backed-off retries to overcome rate limit error
                                                                                        # similarly other supabase interactions, now get this to use supabase core methods module
                                                                                        # redo with dictionary returns for methods
            if not isinstance( sb_client, sb_package.Client):
                raise RuntimeError(f'could not get sb client instance')                 # exception signals to, and is picked up in, SessionManager

            response  = sb_client.auth.sign_in_with_password( {"email":    email,            # fresh login
                                                               "password": password} )     
            self.latest_client = sb_client
                                                              
        except Exception as ex:
            msg = f"Error creating '{self.name}' {email}:\n\n\tXXXXX { str(ex)} XXXXX\n\n"
            logger.error(  msg)
            raise RuntimeError( msg)                                                    # exception signals to, and is picked up in, SessionManager
            
                        
        sb_session = response.session        
        if sb_session:                
            self.set_object_attribs( sb_session) 
        else:
            logger.error(f"Error refreshing '{self.name}': no sb session")

        
    def set_object_attribs( self,  sb_session):
        
        logger.debug(f"setting attribs for session '{self.name}'")
        
        self.access_token    = sb_session.access_token
        self.refresh_token   = sb_session.refresh_token
        
        if self.latest_client is None:
            raise RuntimeError(f'\n\nno client')
        
        self.latest_client.auth.set_session( access_token  = self.access_token,
                                             refresh_token = self.refresh_token  )              # equip the client with a valid set of headers 
                                                     
        now_utc              = datetime.now( timezone.utc)                                      # all internal time comparisons done in UTC 'Zulu time', simples
        expires_in           = sb_session.expires_in          

        delta       = timedelta( seconds=20)   if self.quick_test    else   timedelta( seconds=expires_in)   # in quick test, expire in 20 secs instead of 3600
        self.expires_at_utc  = now_utc  +  delta
        expires_at_local     = str( datetime.now() + delta)
        logger.info(f"New session '{ self.name}' will expire at { expires_at_local[11:19]}")    # display local system time to avoid confusion
                


      
                                                                      
    def refresh( self):
        try:
            response = self.latest_client.auth.refresh_session( self.refresh_token)
            logger.info(f"\n\n\t***** Refreshing session '{self.name}' *****\n\n")
            
        except Exception as ex:
            msg = f"Error refreshing '{self.name}': { str(ex)}"
            logger.error(  msg)
            raise RuntimeError( msg)

        sb_session = response.session
        if sb_session:                
            self.set_object_attribs( sb_session)

        

    def is_expired(self):
        now     = datetime.now( timezone.utc)                                                   # all time comparisons done in UTC
        expiry  = self.expires_at_utc
        expired = now > expiry 
        if expired:
            logger.warning( f"'{self.name}' PrSession has expired")
        return  expired

    

    def is_approaching_expiry(self):
        now         = datetime.now( timezone.utc)
        expiry      = self.expires_at_utc
        delta       = timedelta( seconds=2) if self.quick_test else   timedelta( seconds=60)    # in 60 seconds window before expiry => return True (2 seconds in quick test mode)                  
        approaching = (now + delta > expiry)  and  (now < expiry)
        return  approaching
        
        
        
                    
    '''
    def pretty_print_login_data( self, login_data, session): # purpose is to make json data in the log human readable
        #----------------------------------
        def custom_serializer( obj ):                   # JSON serializer for objects not serializable by default json code        
            if isinstance( obj, datetime ):
                return obj.isoformat()
            if hasattr( obj, '__dict__' ):
                return obj.__dict__
            return str( obj )
        #----------------------------------

        data = {
            "user":    login_data,
            "session": session
        }
        return json.dumps( data, default=custom_serializer,  indent=4 )
        
    '''

        




        
# ------------------------------------------------------------------------------- Pristina Session Manager
   

class SessionManager:
    
    # To properly manage interaction with Supabase from multiple concurrent threads. 
    # Code uses a re-entrant thread lock and 'with' context management for threadsafe operation
    # Refreshing of threads before expiry for long-running sessions is done using an independent timer/checker thread (daemon so shuts itself on program exit)
    # Code ensures that each session can be logged out individually (or say on error, all at once)
    # this should completely overcome the problem of Blender not shutting down fully and running on in the background.

    email        = None
    password     = None
    sessions     = {}
    lock         = threading.RLock()    # recursive locking eg when close all()  calls logout(), each needs to use lock
    exp_thread   = None                 # expiry checker background thread, regularly checks sessions for approaching expiry
    quick_test   = False
    
    
    @staticmethod
    def  get_thread_id_with_name( name):                                        # utility to ensure consistency in naming of sessions by thread and by given name
        this_thread_id = str(threading.current_thread().ident)                  # Get the current thread's identifier
        this_thread_id += '--' + name                                           # Append the provided name to the thread identifier
        return this_thread_id


    @classmethod
    def setup( cls, email, password):    
        cls.email    = email
        cls.password = password        
        if cls.exp_thread is None:                                              # Start daemon thread to check for expiry_checker() 
            cls.exp_thread = threading.Thread( target = cls.expiry_checker, 
                                               daemon = True)                   
            cls.exp_thread.start()
        logger.info(f'\n\n\t***** SessionManager setup for {email} \n\texpiry checker thread started\n\n')
        
        
    def change_credentials( cls, email, password):
        cls.email    = email
        cls.password = password        


    @classmethod
    def get_client( cls, name):     # previously (doing the coding) we got the auth'd client first, 
                                    # now get a session which may be refreshed later (ie the Client object changes), 
                                    # and make a fresh current usable client from that, at the time of need, 
                                    # so guaranteed to send a header with valid tokens
                                    # otherwise Pristina may stop working after exactly 3600 seconds (1 hour)
                                    # alternative is to let sessions expire and then re-login when next needed.
                                    # commentators say this method can give bad user experience, 
                                    # as login can sometimes take longer than expected, disrupting smooth usage
                                         
        # logger.info(f"for '{name}' session")
        
        with cls.lock:
            client_already_exists = (name in cls.sessions )
            
            if  client_already_exists   and    not cls.sessions[name].is_expired():                    
                pr_session = cls.sessions[name]                                 # existing session is fine              
                logger.debug(f"for '{name}' using existing session") 
                
            else:    
                try:
                    pr_session = _PrSession( cls.email, cls.password, name=name,  quick_test=cls.quick_test)   # not got already, make fresh new 
                except:
                    return None     # credentials failed for sign in
                    
                logger.debug(f"adding new session to dictionary for '{name}'") 
                cls.sessions[name] = pr_session
                  
        return  pr_session.latest_client
        
        
        
        

    @classmethod
    def expiry_checker(cls):                                                    # ********** RUNS IN DAEMON THREAD (IN BACKGROUND) ******
        check_every = 2   if cls.quick_test    else  60        
        while True:
            with cls.lock:                
                if len(cls.sessions) > 0:
                    logger.info(f'------------------------  expiry checking on { len(cls.sessions)} sessions\n{cls.sessions.keys()}')
                    
                for name in cls.sessions:                                       # Check each session in the session manager   'for each key:'
                    pr_session = cls.sessions[name]                             # lookup value, ie stored session ref
                    if pr_session.is_approaching_expiry():                      # Refresh the access token if it's about to expire
                        pr_session.refresh()
                        
            time.sleep(check_every)                                             # Sleep before checking again
            
        
        
    @classmethod
    def logout( cls, name):
        # logger.info(f"logging out '{name}'")
        with cls.lock:
            if name in cls.sessions:
                pr_session  = cls.sessions[name]
                try:
                    pr_session.latest_client.auth.sign_out()                    # ESSENTIAL TO CALL SIGN_OUT ON ALL LIVE SESSIONS AT END, 
                                                                                # ELSE RUN HANGS / GOES INTO BACKGROUND   
                    del cls.sessions[name]                                                                                
                    logger.info(f"logged out of '{name}'   ({len(cls.sessions)} sessions remain)")

                except Exception as ex:
                    logger.exception( f"Exception logging out '{name}'")
                                
            else:
                if not 'Main' in name:                                          # ok to have say '13004--Main' not found, 
                    logger.warning(f"'{name}' not found in mgr")                # as now do early logout after fetch credits
                                                                                # todo make a diagram and double-check the lifecycle of 'Main'
                
                

    @classmethod
    def close_all_sessions( cls):
        logger.info( f'Closing all sessions')
        with cls.lock:
            sessions_copy = dict( cls.sessions)                                 # Create shallow copy of the dictionary. References to the
                                                                                # original objects are retained in the new structure 
                                                                                # which is what we want, 
                                                                                # for example when logout() does del cls.sessions[name]
                                                                                # if we did not use a copy, 
                                                                                # and worked on the original cls.sessions dictionary,
                                                                                # then as python started on the 'for' loop, 
                                                                                # it would raise an exception, complaining
                                                                                # the dict. size was changing during the loop            
            
            
            logger.info(f'got lock, now closing {len( cls.sessions)} sessions')
            for name in sessions_copy:
                logger.info(f"Closing session '{name}'")
                cls.logout( name)                                               # call logout() for each session in dict.
   
        cls.sessions.clear()                                                    # clear the dictionary
        logger.info(f'All logged out & dictionary emptied')


'''        
        # cls.force_cleanup_ssl()                                               # Force cleanup of dangling SSL sockets
                                                                                # can happen, likely due to Supabase client
                                                                                # could affect other programs, comment out
        # for meantime --- if we could know the sockets used by the sb_client, then could close them down selectively
        # ssl has only been a problem in testing so far, not an issue in production, 
        # ie in normal use, ssl sockets are all closed ok
        




    @staticmethod
    def force_cleanup_ssl():
        """
        Force cleanup of unreferenced SSL sockets.
        """
        logger = Logger.get_logger()
        for obj in gc.get_objects():
            if isinstance(obj, ssl.SSLSocket):
                try:
                    obj.close()
                    logger.info( "Closed unreferenced SSL socket.")
                except Exception as ex:
                    logger.warning( f"Error closing SSL socket: {ex}")
        
        gc.collect()                                                            # Invoke garbage collection

'''                
                
                
      


##SessionManager.setup( "neal@irlabs.co.uk", "123456789")  #!!!!!!!!!!!!!! temp todo integrate into main thread after user logs in ui






# todo --- remember password is not secure -- clear it after being accepted, have user login each time ? 
# or give option to save on machine (eg. in environment vars), so is convenient for user, 
# but not exposed (or hidden but copyable in ui) ?


