# task_manager.py

# from task_manager import TaskManager


from logger_module                  import   Logger
logger       = Logger.get_logger()      ####################  global, is set up on load 



class Task:
    def __init__( self, taskId, name, status='PENDING'):        #  Tasks are created in the initial pending state
        self.taskId = taskId                                    #  they have  id, status, and optional name
        self.status = status
        self.name   = name
        self.color  = (0.0, 0.0, 1.0, 1.0)  # Blue for PENDING
        logger.info( f"Task:  new task id {taskId} 'PENDING'")

        
        

class TaskManager:
    
    task_lookup = {}                                            # Class-level attributes
    
    status_colors = {                                           # dropdown traffic light colors RGBA
        'PENDING'  : (0.0, 0.0, 1.0, 1.0),          # blue
        'ACTIONED' : (1.0, 1.0, 0.0, 1.0),          # yellow
        'COMPLETE' : (0.0, 1.0, 0.0, 1.0),          # green
        'FAIL'     : (1.0, 0.0, 0.0, 1.0)           # red
    }
    indicator_colors = {                                        # next to Render button main indicator
        'default'     : (0.44, 0.0, 0.9, 1.0),      # violet/purple
        'credit_check': (1.0,  0.5, 0.0, 1.0),      # orange
        'in_render'   : (1.0,  0.0, 0.0, 1.0),      # red
        'in_progress' : (1.0,  1.0, 0.0, 1.0),      # yellow (1 or more task has status PENDING or ACTIONED
        'FAIL'        : (1.0,  0.0, 0.0, 1.0)       # poss use to warn server is down
    }
    current_indicator_color = indicator_colors['default']
    
    

    @classmethod
    def add_task( cls, task_obj):                                # initially the whole task obj is added here
        cls._check_and_update_indicator_color()
        if task_obj.taskId in cls.task_lookup:
            return "Task with ID already exists."
        else:
            cls.task_lookup[task_obj.taskId] = task_obj
            return f"Task {task_obj.taskId} added as '{task_obj.name}'."
        
        

    @classmethod
    def update_task_status( cls, taskId, status):                # update and remove only need telling the taskId
        logger.info( f'TaskManager:  updating status {taskId} as {status}')
        if taskId in cls.task_lookup:
            logger.debug( f'TaskManager:  found {taskId} in lookup dict')
            task        = cls.task_lookup[taskId]
            task.status = status
            task.color  = cls.status_colors[status]            
            cls._check_and_update_indicator_color()                        
            return f"Task {taskId} status updated to {status}."
        
        else:
            if status == 'PENDING':                             # then new job, new entry needed
                name            = ''     # do not use unless/until fully implemented 
                                                                # (will need 3 element list on fifo not 2)
                                                                
                task_obj        = Task( taskId, name, status='PENDING')
                task_obj.color = cls.status_colors[status]     # NB task color and indicator color are 2 different things    
                cls.add_task( task_obj)            
                cls._check_and_update_indicator_color()                        
                return "Task ID added to manager."
            else:
                return(f'args {taskId}, {status} inconsistent error')
        
    
    
    @classmethod
    def count_all_pending_and_actioned( cls):                   # value to show number of jobs in progress
        count = 0
        for task in cls.task_lookup.values():
            if task.status in ('PENDING', 'ACTIONED'):
                count += 1
        return count
    
    
    
    @classmethod
    def _check_and_update_indicator_color( cls):
        from nested_ui_panels   import PRISTINA_PT_MainPanel

        num_in_progress = cls.count_all_pending_and_actioned()
        if num_in_progress == 0:
            cls.current_indicator_color = cls.indicator_colors['default']
        else:
            cls.current_indicator_color = cls.indicator_colors['in_progress']
        PRISTINA_PT_MainPanel.redraw_panel()
        logger.debug(f'jobs={num_in_progress}   colour={cls.current_indicator_color}')

                
    
    @classmethod
    def set_indicator_color( cls, situation):     
        from nested_ui_panels   import PRISTINA_PT_MainPanel

        cls.current_indicator_color = cls.indicator_colors[ situation]   # eg 'in_render'
        PRISTINA_PT_MainPanel.redraw_panel()

                
    

    @classmethod
    def remove_task( cls, taskId):
        if taskId not in cls.task_lookup:
            return "Task with ID not found."
        else:
            del cls.task_lookup[taskId]
            cls._check_and_update_indicator_color()            
            return f"Task {taskId} removed."
        


