# upload_task_wait_complete.py

# from upload_task_wait_complete   import upload_task

import bpy
import os
import time
import threading
import shutil
import fifo_queue_module
from datetime          import   datetime, timedelta
from logger_module     import   Logger
from sessions          import   SessionManager
from file_utils        import   append_timestamp_to_filename,   get_md5,   download_and_unzip

logger       = Logger.get_logger()      ####################  global, is set up on load 



supabase_storage_url = "https://fdqpbqadouajjpqluikg.supabase.co/storage/v1/object/public/pristina_results/"



# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# !!!!!!!!! this method starts up in its own new thread, 
# !!!!!!!!! it is not allowed to do main-thread UI updates directly, it *must* use the fifo queue to send messages
# It sets up its own supabase auth session (and corresponding client)
# It uses the SessionManager to login, jwt refresh before expiry, to provide up to date client for supabase calls, 
# and makes sure to logout of each individual thread.
# IF ANY LOGOUT IS MISSED, THEN THE PROGRAM HANGS -- DOES NOT FINISH PROPERLY, AND CARRIES ON RUNNING AS A BACKGROUND TASK
# THIS CAN BE STOPPED WITH WINDOWS TASK MANAGER IF IT HAPPENS, BUT IT SHOULD NOT.



# called from fifo_queue_module

def upload_task(  pristinaFilePath,  userEmail,  userPassword,  context, fifo_queue,  timeout_seconds):
    
    
    
    if SessionManager.email is None:
        SessionManager.setup( userEmail, userPassword)
        
        
    base_name = os.path.basename( pristinaFilePath)
    jobname   = os.path.splitext( base_name)[0]              # just the filename without the .pristina extension
    
    logger.info(f'base_name={base_name}  jobname={jobname}')
    
    this_thread_id  = SessionManager.get_thread_id_with_name( jobname[-9:] )   # using job timestamp to get 'threadId--dd_hhmmss'

    taskId    = None      # has this value until number received back from supabase insert
    
    try:

        sb_client  = SessionManager.get_client( this_thread_id)     # to get to here,  SessionManager 
                                                                    # will already be set up and have email,pxwd                
        
        response         = sb_client.auth.get_user()
        user             = response.user

        if user:
            logger.info( f"job thread {this_thread_id} Auth ok!")
            file_name    = append_timestamp_to_filename( os.path.basename( pristinaFilePath))
            
            md5          = get_md5( pristinaFilePath)               # used by processor to detect duplicate jobs
            
            storage_path = f"pristina/{file_name}"
            

            with open(pristinaFilePath, "rb") as file:
                file_content = file.read()
            
            # Attempt to upload the file
            upload_result    = sb_client.storage.from_("pristina_files").upload( storage_path, file_content)
            logger.info( f"Upload result: {upload_result}")
            
            
            # NB the following data insert failed unexpectedly end of Nov 2024 and was subject of support emails
            # then it came back working again.  Did Supabase have an update problem that they fixed after a few days??
            

            # Insert entry into 'task_queue' table
            data = {
                "user_email": userEmail,
                "user_id":    user.id,
                "status":     "PENDING",
                "pristina":   storage_path,
                "md5":        md5
            }
            insert_result    = sb_client.table("task_queue").insert(data).execute() # insert the new task in Supabase            
            taskId           = insert_result.data[0]['id']                          # get taskId back from Supabase
            
            setup_named_output_folder( context, pristinaFilePath, taskId)           # name and taskId 'saved' in case plugin shut down
            
            
            this_thread      = threading.current_thread()               # get current thread, and
            this_thread.name = f'pristina_thread_id_{ taskId}'          # now we know the taskId, we can *** NAME THIS THREAD ***
            
            
            logger.info(f'\n\n\t***** named thread as    "{this_thread.name}"     task id={taskId} *****\n\n')            
            
            # ''''''''''''''''''''''''''''''''''''''''''''''''''''''''

            wait_for_complete( taskId, fifo_queue, this_thread_id,  timeout_seconds)  # *** BLOCKS *** 
                                                                                                    # Waits for task completion (or fails)

            # ''''''''''''''''''''''''''''''''''''''''''''''''''''''''

            SessionManager.logout( this_thread_id)                     
            
            return "success"  # ** todo when do retrex, convert this to dict return according to wait_for_complete() return status
                              

        else:
            logger.warning( f"'{this_thread.name}'  Authentication failed, no user found")  
            
            #context.scene.pristina_props.auth_status = False           # todo? Set auth_status to False, 
                                                                        # ui should update to this itself?      
            return "failure"



    except Exception as e:
        logger.exception( f"\n\nError in upload task: {str(e)}\n\n")    # logger exception should give traceback
        if taskId:
            fifo_queue.put(  [taskId, 'FAIL']  )                      # q  as FAIL
        
        return "failure"    # todo  these return values are no longer used, all messaging is now done in fifo_queue and in log
        
 

def get_task( sb_client, taskId):
    # logger.info( f"listening for completed job { taskId}")
    response = sb_client.table("task_queue").select("*").eq("id", taskId).execute()
    task = response.data[0]
    return task
    
    
    
def setup_named_output_folder( context, pristinaFilePath, taskId):    # todo remove pfp from args

            
    # starting new job cycle, first thing, setup the output folder using the user's own jobname and task_id
    # so if plugin is shutdown before pristina rendering in complete, then the name of the folder
    # will remember the job name, and the when the job is later fetched, the results can be 
    # married up by matching the task_id of the job to that mentioned in the folder name
    
    '''
    logger.critical(f'pristinaFilePath={pristinaFilePath}')
    # warning, path may include relative addressing eg  C:\ Users\cex\Downloads\..\Documents\Pristina_tmp\output_20241104_153738.pristina
    # need to regularize
    
    pristinaFilePath = os.path.normpath( pristinaFilePath )                     # remove any relative addressing in path eg xx\..\yy

    if pristinaFilePath.startswith("//"):        
        abspath = bpy.path.abspath( pristinaFilePath)
    else:        
        abspath = pristinaFilePath                                              # It's already an absolute path  
                                                                                # eg "C:\\ Users\\cex\\Documents"    
                                                                                # (space inserted to stop unicode errors)
    '''
                                                                                
    local_path = bpy.context.scene.pristina_properties.file_output
    if local_path.startswith("//"):        
        abspath = bpy.path.abspath( local_path)
    else:        
        abspath = local_path                        # It's already an absolute path  eg "C:\\Users\\cex\\Documents"
        
    local_path = os.path.normpath( abspath )        # remove any .. relative addressing  eg aa\xx\..\yy --> aa\yy


    
    user_jobname = context.scene.pristina_properties.job_name    

    if user_jobname == '':       # todo see file utils around line 150  redundancy (will be)
        user_jobname = 'Job'     # default if user has not filled the 'Job Name' field in the UI for this task
                            
    this_job_results_folder  =  os.path.join( local_path,  user_jobname + "___ref_"  +  str( taskId))
    
    logger.critical(f'this_job_results_folder={this_job_results_folder}')
    
    os.makedirs( this_job_results_folder, exist_ok=True )                       # setup the results folder before processing

    


#py# wait_for_complete

# !!!!!!!!!!! THIS METHOD RUNS IN ITS OWN JOB THREAD (along with 'UPLOAD TASK()' which is this method's caller), 
#             IT TALKS TO THE MAIN THREAD (*ONLY*!!) BY PUTTING MESSAGES ON THE FIFO QUEUE (WHICH IS THEN THREADSAFE)

def wait_for_complete( taskId,  fifo_queue,  this_thread_id,  timeout_seconds):
    
    from ot_render_pristina  import  PRISTINA_OT_RenderPristina                 # used here for messages, 
                                                                                # as show method needs context window, import technique
                                                                                # used to break out of circular import references
     
    try:        
        status      = 'PENDING'                                                 # set initial state, same for all jobs        
        last_status =  status                                                   
        
        fifo_queue.put( [taskId, status])                                       # MUST NOT TALK DIRECT TO TASKMANAGER 
                                                                                # -- ONLY MESSAGE VIA FIFO QUEUE FOR THREAD SAFETY
                                                                                # q  new PENDING, will setup new task in TaskManager, 
                                                                                # with its own timeout calculated on the number of items submitted
                                                                                
        actioned_at   = None                                                    # set when status goes to ACTIONED, 
                                                                                # declare here to ensure scope
            
        while not fifo_queue_module.stop_event.is_set():
            
            sb_client = SessionManager.get_client( this_thread_id)              # get the latest client from manager, possibly refreshed
            task      = get_task( sb_client, taskId)                            # in loop checking for status changes in database
            status    = task[ 'status']
            
            if status in ['ACTIONED', 'FAIL']:
                if last_status != status:             
                    
                    if status == 'ACTIONED':                        
                        actioned_at = datetime.now()                            # Set timestamp the moment the job is Actioned
                        logger.info(f'actioned_at={actioned_at}')               # job must complete within timeout_seconds from here
                                        
                    fifo_queue.put(  [taskId, status]  )                        # q  as ACTIONED or FAIL
                    logger.info(  f"wait_for_complete:  Task {taskId} new status={status}.")
                    last_status = status
                if status == "FAIL":                     
                    logger.error(  f"Task {taskId} new status={status}.")
                    return

            if status == "COMPLETE":
                logger.info(  f"Task is COMPLETE.")                             # job is complete and results zip file can be downloaded
                localpath = bpy.context.scene.pristina_properties.file_output
                logger.info( f'localpath = {localpath}')
                logger.info( f'task      = {task}')
                storage_path   = task['render']['zip']                          # get the zip's bucket storage path 
                                                                                # from supabase render column (json)
                logger.info(f'storage_path={storage_path}')
                                      
                # ''''''''''''''''''''''''''''''''''''''''''''''''''''''''
                
                result = download_and_unzip( storage_path, localpath, this_thread_id)     # DOWNLOAD & UNZIP (in file_utils)

                # ''''''''''''''''''''''''''''''''''''''''''''''''''''''''
                
                if result['error']:
                    logger.error( f"{result['message']}")
                    status = 'FAIL'
                    
                fifo_queue.put( [taskId, status])                               # q  COMPLETE (or zip FAIL)
                return
                
            else:
                if fifo_queue_module.stop_event.is_set():                       # if stopping, skip the wait, return immediately
                    break
                    
                now = datetime.now()                                            # *** CHECK FOR JOB TIMEOUT  ***
                if  actioned_at  and   (now >= actioned_at + timedelta( seconds = timeout_seconds)):  
                                        # see: made in ot_render_pristina (per item setting) >> fifo_queue_module >> here 
                    fifo_queue.put(  [taskId, 'FAIL']  )                        # q  as FAIL
                    logger.error(f'\n\n\t***** Task {taskId} timed out, ceasing to check status and wait for completion *****\n\n')                      
                    # PRISTINA_OT_RenderPristina.show_message(f"Task {taskId} failed: (internet connection issue or server temporarily unavailable)")  
                    break
                    
                    
                seconds = 5                                                     # set to 1 second when testing timeout functionality on 
                                                                                # a 10 second timeout (see ot_render_pristina.py for setting that)
                
                logger.info( f"Task {taskId} status is {task['status']}.     Checking again in {seconds} seconds.")
                
                time.sleep( seconds)                                            # Wait before the next check
                
        logger.info(f'stopped for task {taskId}')

    except Exception as ex:
        logger.exception( f'\n\n\t***** wait_for_complete() exception: *****\n\n{ex}\n\nTask failed\n\n\n')
        fifo_queue.put(  [taskId, 'FAIL']  )                                  # q  as FAIL
        SessionManager.logout( this_thread_id) 
        # PRISTINA_OT_RenderPristina.show_message(f"Task {taskId} failed -- see log for details")    
        return



#1#



'''

Task Timeouts explanation

in

ot_render_pristina.py
	   
	when ready to submit new job
	the 'timeout_at' time is calculated and passed to new_job() in fifo_queue_module



fifo_queue_module.py

	new_job()  then starts a new thread to handle the job, and passes 'timeout_at'  
	to upload_task() in the upload_task_wait_complete  module



upload_task_wait_complete.py

	upload_task() method passes 'timeout_at'  to  wait_for_complete()

	and in the wait loop (checking status every 5 seconds) 
	the 'timeout_at' is checked against current time, 
	and then times out if gone over (time now > timeout_at).

	The task status is set to FAIL, and a message is logged
	***** Task {taskId} timed out, ceasing to check status and wait for completion *****
    
    
    And a popup message to the user says
    "Task {taskId} failed \n(internet connection issue or \nserver temporarily unavailable)"




'''
