# viewport_renderer.py


import bpy
import os



class ViewportRenderer:
    def __init__(self, output_path):
        self.output_path                        = output_path
        self.original_settings                  = {}
        self.original_color_management_settings = {}
        
        
    '''    
    ###############  download_matcap code currently redundant, as file is now bundled in plugin zip.  ###################
    ###############  But keep in case need to change back from some reason                            ###################
    
    def download_matcap(self, matcap_url, matcap_name):
    
        matcap_folder = bpy.utils.user_resource('DATAFILES', path="studiolights/matcap", create=True)
        matcap_path   = os.path.join(matcap_folder, matcap_name)

        if not os.path.exists(matcap_path):
            if ensure_requests_package():
                logger.info( f"Downloading matcap: {matcap_name}")
                
                try:
                    #----------------------------------------------
                    with requests.Session() as session:            # cnm 'with' and 'session' intended to make sure all closed after
                        response = session.get( matcap_url)
                        if response.status_code == 200:
                            with open(matcap_path, 'wb') as f:
                                f.write(response.content)
                            logger.info( f"Matcap downloaded to: {matcap_path}")
                        else:
                            logger.error( f"\n\n\n\t>>>>>>>>>>>>>>>>>>>> Failed to download matcap. Status code: {response.status_code}\n\n")
                    #----------------------------------------------
                    
                except requests.RequestException as e:
                    logger.error(f"An error occurred while downloading the matcap: {e}")                
                
            else:
                logger.error( "requests package is not available. Please download the matcap manually.")
        else:
            logger.warning( f"Matcap already exists at: {matcap_path}")
            
    '''

    

    def save_original_settings(self):
        view_settings = bpy.context.scene.view_settings
        self.original_color_management_settings = {
            'view_transform': view_settings.view_transform,
            'look':           view_settings.look,
            'exposure':       view_settings.exposure,
            'gamma':          view_settings.gamma,
        }

        for area in bpy.context.screen.areas:
            if area.type == 'VIEW_3D':
                space_data = area.spaces.active
                self.original_settings[area.as_pointer()] = {
                    'view_perspective':      space_data.region_3d.view_perspective,
                    'shading_type':          space_data.shading.type,
                    'shading_light':         space_data.shading.light,
                    'shading_studio_light':  space_data.shading.studio_light,
                    'show_overlays':         space_data.overlay.show_overlays,
                    'show_gizmo':            space_data.show_gizmo,
                    'show_object_outline':   space_data.shading.show_object_outline,
                    'use_dof':               space_data.shading.use_dof,
                    'show_cavity':           space_data.shading.show_cavity,
                    'show_backface_culling': space_data.shading.show_backface_culling,
                    'show_xray':             space_data.shading.show_xray,
                    'show_shadows':          space_data.shading.show_shadows,
                    'color_type':            space_data.shading.color_type  # Save original color type
                }

    def apply_settings(self):
        self.save_original_settings()
        
        # if 'check_normal_pristina.exr'  is not found when just installed, the only safe way
        # is to restart Blender, there is no studiolights refresh operator...
        
        for area in bpy.context.screen.areas:
            if area.type == 'VIEW_3D':
                space_data = area.spaces.active
                space_data.region_3d.view_perspective    = 'CAMERA'
                space_data.shading.type                  = 'SOLID'
                space_data.shading.light                 = 'MATCAP'
                space_data.shading.studio_light          = 'check_normal_pristina.exr'
                space_data.overlay.show_overlays         = False
                space_data.show_gizmo                    = False
                space_data.shading.show_object_outline   = False
                space_data.shading.use_dof               = True
                space_data.shading.show_cavity           = False
                space_data.shading.show_backface_culling = False
                space_data.shading.show_xray             = False
                space_data.shading.show_shadows          = False
                space_data.shading.color_type            = 'OBJECT'  # Set color type to OBJECT
        bpy.context.scene.view_settings.view_transform   = 'Standard'
        bpy.context.scene.view_settings.exposure         = 0
        bpy.context.scene.view_settings.gamma            = 1

    def revert_settings(self):
        view_settings = bpy.context.scene.view_settings
        for setting, value in self.original_color_management_settings.items():
            setattr(view_settings, setting, value)

        for area in bpy.context.screen.areas:
            if area.type == 'VIEW_3D' and area.as_pointer() in self.original_settings:
                original_area_settings = self.original_settings[area.as_pointer()]
                space_data             = area.spaces.active
                space_data.region_3d.view_perspective = original_area_settings['view_perspective']
                for key, value in original_area_settings.items():
                    if 'shading_' in key and hasattr(space_data.shading, key.replace('shading_', '')):
                        setattr(space_data.shading, key.replace('shading_', ''), value)
                    elif key in ['show_overlays', 'show_gizmo'] and hasattr(space_data.overlay, key):
                        setattr(space_data.overlay, key, value)
                    elif key == 'show_object_outline' and hasattr(space_data.shading, 'show_object_outline'):
                        space_data.shading.show_object_outline = value
                    elif key in ['use_dof', 'show_cavity', 'show_backface_culling', 'show_xray', 'show_shadows'] and hasattr(space_data.shading, key):
                        setattr(space_data.shading, key, value)
                    elif key == 'color_type' and hasattr(space_data.shading, 'color_type'):
                        space_data.shading.color_type = value  # Revert color type

    def render_screen_preview(self):
        os.makedirs(os.path.dirname(self.output_path), exist_ok=True)
        bpy.context.scene.render.filepath = self.output_path
        bpy.ops.render.opengl(write_still=True)
        self.revert_settings()

    #def run(self, matcap_url, matcap_name):
        #if ensure_requests_package():
            #self.download_matcap(matcap_url, matcap_name)

    def run(self):        
        self.apply_settings()
        self.render_screen_preview()